# Copyright start
# Copyright (C) 2008 - 2021 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end
source_docker_utils() {
    local fsr_docker_utils_sh="/opt/cyops/configs/scripts/docker/utils.sh"
    source $fsr_docker_utils_sh
}

init() {
    source_docker_utils
    prefix="/opt/cyops-api"
}

validate_input() {
    local context=$1
    local version=$2
    if [ "X$context" = "X" ] || [ "X$version" = "X" ]; then
        echo "$0 upgrade/common <version-here>"
        exit 1
    fi
}

update_proxy() {
    local conf="/etc/php-fpm.d/cyops-api.conf"
    if grep -q "PROXY" $conf; then
        # Since proxy is already added, return
        return
    fi
    echo "" >>$conf
    echo "env[HTTPS_PROXY]= \$https_proxy" >>$conf
    echo "env[HTTP_PROXY]= \$http_proxy" >>$conf
    echo "env[NO_PROXY]= \$no_proxy" >>$conf
}

handle_docker_upgrade() {
    #added since we are changing the encryption key , can be removed once the 7.0.1 support is obselete
    mv /opt/cyops/configs/cyops-api/.Defuse.key /opt/cyops/configs/cyops-api/.Defuse_cloud.key
    sudo -u nginx php /opt/cyops-api/bin/console doctrine:schema:update --force
    sudo -u nginx php /opt/cyops-api/bin/console doctrine:decrypt:database --no-interaction --env=prod
    mv /opt/cyops/configs/cyops-api/.Defuse_cloud.key /opt/cyops/configs/cyops-api/.Defuse.key.bkp
    sudo php /opt/cyops-api/bin/generate-fsr-defuse-key >/dev/null
    chown nginx:nginx /opt/cyops/configs/cyops-api/.Defuse.key
    chmod 644 /opt/cyops/configs/cyops-api/.Defuse.key
    #------
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear
    sudo -u nginx php /opt/cyops-api/bin/console app:metadata:generate:models
    # Need to rerun cache:clear as initially the files are generated based on 7.0.1 (latest release code) and then files are reverted back to original code
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear
    sudo -u nginx php /opt/cyops-api/bin/console doctrine:schema:update --force
    sudo -u nginx php /opt/cyops-api/bin/console doctrine:migrations:migrate --no-interaction
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear
    sudo -u nginx php /opt/cyops-api/bin/console cache:warmup --no-debug
    sudo -u nginx php /opt/cyops-api/bin/console app:system:update -lafc
}
handle_rpm_upgrade() {
    #added since we are changing the encryption key , can be removed once the 7.0.1 support is obselete
    sudo php /opt/cyops-api/bin/generate-fsr-defuse-key >/dev/null
    chown nginx:nginx /opt/cyops/configs/cyops-api/.Defuse.key
    chmod 644 /opt/cyops/configs/cyops-api/.Defuse.key
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear
    #------

    sudo -u nginx php /opt/cyops-api/bin/console doctrine:schema:update --force
    sudo -u nginx php /opt/cyops-api/bin/console doctrine:migrations:migrate --no-interaction
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear
    sudo -u nginx php /opt/cyops-api/bin/console cache:warmup --no-debug
    sudo -u nginx php /opt/cyops-api/bin/console app:system:update -lafck
}

handle_upgrade_steps() {
    local new_version=$1

    # Remove Temporary Created Person and Appliance Entities
    rm -rf /opt/cyops-api/src/Entity/Actors/Appliance.php
    rm -rf /opt/cyops-api/src/Entity/Actors/Person.php

    # Remove Post 7.0.2 release.
    # Done for removing warnings.
    rm -f /opt/cyops-api/src/Command/WidgetsBuildCommand.php
    rm -rf /opt/cyops-api/var/cache/prod

    # Write post upgrade action here

    ln -sf /opt/cyops/configs/keys/APPLIANCE_PUBLIC_KEY ${prefix}/src/keys/appliance_public.key
    ln -sf /opt/cyops/configs/keys/APPLIANCE_PRIVATE_KEY ${prefix}/src/keys/appliance_private.key

    python3 /opt/cyops/configs/scripts/confUtil.py -f /opt/cyops/configs/cyops-api/mergeConfig.yml

    # Enable Cache Clear for Drop MetaData Command to be Available in 5.2
    sudo -u nginx php /opt/cyops-api/bin/console cache:clear

    sudo -u nginx php /opt/cyops-api/bin/console app:drop:metadata:copy

    sleep 8
    restorecon -R ${prefix}

    # Run full commands for cyops-api packages to generate new classes and tables
    sudo chown -R nginx:nginx /opt/cyops-api/src/Entity/Custom/
    if is_docker; then
        handle_docker_upgrade
    else
        firewall-cmd --reload
        handle_rpm_upgrade
    fi

    restorecon -R ${prefix}
    # restart services
    systemctl daemon-reload
    systemctl restart nginx.service
}

handle_common_steps() {
    local new_version=$1
    sleep 5
    # Perf Improvement
    sudo -u nginx php /opt/cyops-api/bin/console cache:warmup --no-debug

    if ! grep -q "EnvironmentFile=/etc/environment" /usr/lib/systemd/system/php-fpm.service; then
        sed -i '/Type=notify/a EnvironmentFile=/etc/environment' /usr/lib/systemd/system/php-fpm.service
        sed -i 's/PrivateTmp=true/PrivateTmp=no/g' /usr/lib/systemd/system/php-fpm.service
    fi

    update_proxy

    # reload php-fpm.service for EnvironmentFile changes
    systemctl daemon-reload
    systemctl restart php-fpm.service

    # Set selinux user system_u
    semanage fcontext -a -s system_u -t var_log_t "/var/log/cyops/cyops-api"

    # touch file so that restorecon can set the right selinux permissions
    sudo -u nginx touch /var/log/cyops/cyops-api/last_system_publish.log
    sudo -u nginx touch /var/log/cyops/cyops-api/last_config_export.log
    sudo -u nginx touch /var/log/cyops/cyops-api/last_config_import.log
    restorecon -rF /var/log/cyops/cyops-api
    restorecon -rF /opt/cyops-api
}

############
# Main/main/MAIN starts here
#########
context=$1
version=$2
init
# This script usually runs from the specfile or docker upgrade. Hence add -x here for debug.
set -x
validate_input "$context" "$version"
case $context in
upgrade)
    handle_upgrade_steps "$version"
    ;;
common)
    handle_common_steps "$version"
    ;;
esac
exit 0
