<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

class AppCacheClear extends Command
{
    private $container;
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        parent::__construct();  
    }

    protected function configure()
    {
        $this
            ->setName('app:cache:clear')
            ->setAliases(['redis:flushdb']) // for backward compatibilty. lot of places in other services flush redis cache using this
            ->setDescription('This command allows you to clear the in memory cache');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        opcache_reset();
        $headers = array(
            "Content-type" => "application/json",
        );
        $method = 'POST';
        $uri = sprintf('%s/api/3/cache_util', $this->container->getParameter('crudhub.uri'));
        $data = ['operation' => 'clear-cache'];
        $client = $this->container->get("app.proxy.client.crudhub");
        $request = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
        $response = $client->send($request, ['verify' => false]);
        $statusCode = $response->getStatusCode();
        if ($statusCode != Response::HTTP_OK) {
            $io->success('APCu Cache cleared');
            return -1;
        }
        $io->success('APCu Cache cleared');
        return 0;
    }
}
