<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Conditional;

use App\Conditional\ConditionSet;
use App\Conditional\QueryInterface;
use App\Conditional\QueryMatcher;

/**
 * Class ConditionSetEvaluator
 *
 * @package App\Validation
 */
class ConditionSetEvaluator
{
    /** @var QueryMatcher */
    protected $queryMatcher;

    public function __construct(QueryMatcher $queryMatcher)
    {
        $this->queryMatcher = $queryMatcher;
    }
    /**
     * @param $object
     * @param ConditionSet $conditionSet
     *
     * @return array
     */
    public function evaluate($object, ConditionSet $conditionSet)
    {
        $resultSet = [];

        foreach ($conditionSet->getFields() as $fieldName => $fieldValidations) {
            $attributeName = null;

            foreach ($fieldValidations as $attributeName => $attributeValues) {
                if (!is_array($attributeValues)) {
                    $result = $attributeValues;
                    $resultSet[$fieldName][$attributeName] = $result;
                    continue;
                }

                foreach ($attributeValues as $attributeValue) {
                    if ($attributeValue instanceof QueryInterface) {
                        $matches = $this->queryMatcher->matches($object, $attributeValue);
                        if ($matches) {
                            $result = $attributeValue->getResult();
                            $resultSet[$fieldName][$attributeName] = $result;
                            break;
                        }
                    }
                }
            }
        }

        return $resultSet;
    }

}
