<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Service\IriHelper;
use App\Service\MetadataHelper;
use App\Service\UtilityService;
use App\Security\Voter\FsrVoter;
use App\Controller\BaseController;
use ApiPlatform\Core\Util\CloneTrait;
use Doctrine\ORM\Mapping\ClassMetadata;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use App\Providers\PermissionsDictionaryBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use ApiPlatform\Core\DataProvider\CollectionDataProviderInterface;
use ApiPlatform\Core\Serializer\SerializerContextBuilderInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class AssociationController extends BaseController
{
    use CloneTrait;

    /** @var FsrVoter */
    private $fsrVoter;

    /** @var IriHelper */
    private $iriHelper;

    /** @var  EntityManagerInterface */
    private $entityManager;

    /** @var HttpKernelInterface */
    private $kernel;

    private $utilityService;

    protected $collectionDataProvider;

    protected $serializerContextBuilder;

    protected $inflector;

    public function __construct(
        ContainerInterface $containers,
        ResourceClassResolverInterface $resourceClassResolver,
        FsrVoter $fsrVoter,
        IriHelper $iriHelper,
        EntityManagerInterface $entityManager,
        HttpKernelInterface $kernel,
        MetadataHelper $metadataHelper,
        CollectionDataProviderInterface $collectionDataProvider,
        SerializerContextBuilderInterface $serializerContextBuilder,
        TagAwareCacheInterface $cache,
        UtilityService $utilityService
    ) {
        $this->fsrVoter = $fsrVoter;
        $this->iriHelper = $iriHelper;
        $this->entityManager = $entityManager;
        $this->kernel = $kernel;
        $this->collectionDataProvider = $collectionDataProvider;
        $this->serializerContextBuilder = $serializerContextBuilder;
        $this->utilityService = $utilityService;
        $this->inflector = InflectorFactory::create()->build();
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    public function getAssociationResult(Request $request, $resourceShortName, $id, $field)
    {
        $queryLimit = $request->get('$limit', 30);
        $queryPage = $request->get('$page', 1);

        $resourceClass = $this->getResourceFromClassName($resourceShortName);
        $module = $this->metadataHelper->getModuleFromClassName($resourceClass);

        $this->fsrVoter->denyUnlessAccesssGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . $module);
        $item = $this->iriHelper->findOrThrowNotFound('/api/3/' . $resourceShortName . '/' . $id);

        $this->utilityService->ValidateOwnerShip($request, $item);

        // Check Field Level Permissions
        $this->fsrVoter->denyUnlessAccesssGranted(PermissionsDictionaryBuilder::PERM_FIELD_READ . '.' . $module . '.' . $field);

        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass($resourceClass);
        $classMetadata = $manager->getClassMetadata($resourceClass);
        if ($classMetadata->hasAssociation($field)) {
            /** @var ClassMetadata $classMetadata */
            $association = $classMetadata->associationMappings[$field];
            if (in_array($association['type'], [ClassMetadata::MANY_TO_MANY, ClassMetadata::ONE_TO_MANY, ClassMetadata::TO_MANY])) {
                if ($association['isOwningSide']) {
                    $inversedField = $association['inversedBy'];
                    if (is_null($inversedField)) {
                        $inversedField = $this->metadataHelper->getTypeFromClassName($association['sourceEntity']);
                    }
                } else {
                    $inversedField = $association['mappedBy'];
                    if (is_null($inversedField)) {
                        $inversedField = $this->metadataHelper->getTypeFromClassName($association['sourceEntity']);
                    }
                }
                $targetEntity = $association['targetEntity'];
                $associationResource = $targetEntity;
                $associationModule = $this->metadataHelper->getModuleFromClassName($associationResource);

                $request->query->set($inversedField, $item->getUuid());
                if ($this->containers->hasParameter('ignore_rbac_for_related_record') && $this->containers->getParameter('ignore_rbac_for_related_record')) {
                    $GLOBALS['__associatedFieldRequest__'] = true;
                }
                $request->overrideGlobals();
                $this->fsrVoter->denyUnlessAccesssGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . $associationModule);
                $attributes = [];
                $attributes['filters'] = ['$limit' => $queryLimit, '$page' => $queryPage];
                $attributes['resource_class'] = $associationResource;
                $attributes['collection_operation_name'] = "get";
                $context = $this->serializerContextBuilder->createFromRequest($request, true, $attributes);
                $context['ignore_authorization'] = true;
                $associatedEntities = $this->collectionDataProvider->getCollection($associationResource, $request, $attributes);
                $data1 = $this->get('serializer')->normalize($associatedEntities, 'jsonld', $context);
                return JsonResponse::create($data1);
            }
        }
        throw new BadRequestHttpException("There is no field with the given name in the resource class");
    }

    /**
     * @param Request $request
     * @param string $id
     * @param string $field
     * @param string $fieldId
     * @return Response
     */
    public function deleteFieldAction(Request $request, $resourceShortName, $id, $field, $fieldId)
    {
        $resourceClass = $this->getResourceFromClassName($resourceShortName);
        $module = $this->metadataHelper->getModuleFromClassName($resourceClass);
        $this->fsrVoter->denyUnlessAccesssGranted(PermissionsDictionaryBuilder::PERM_CRUD_DELETE . '.' . $module);

        /** @var BaseEntity $object */
        $item = $this->iriHelper->findOrThrowNotFound('/api/3/' . $resourceShortName . '/' . $id);
        $this->utilityService->ValidateOwnerShip($request, $item);

        $managerRegistry = $this->container->get('doctrine');
        $modelManager = $managerRegistry->getManagerForClass($resourceClass);
        $classMetadata = $modelManager->getClassMetadata($resourceClass);
        if ($classMetadata->hasAssociation($field)) {
            /** @var ClassMetadata $classMetadata */
            $association = $classMetadata->associationMappings[$field];
            if (in_array($association['type'], [ClassMetadata::MANY_TO_MANY, ClassMetadata::ONE_TO_MANY, ClassMetadata::TO_MANY])) {
                $targetEntity = $association['targetEntity'];
                $associateResource = $this->metadataHelper->getTypeFromClassName($targetEntity);
                $associatedModule = $this->metadataHelper->getModuleFromClassName($targetEntity);
                $this->fsrVoter->denyUnlessAccesssGranted(PermissionsDictionaryBuilder::PERM_CRUD_DELETE . '.' . $associatedModule);

                $associatedObject = $this->iriHelper->findOrThrowNotFound('/api/3/' . $associateResource . '/' . $fieldId);
                $this->utilityService->ValidateOwnerShip($request, $associatedObject);
                $fieldName = $association['fieldName'];
                $methodName = $this->inflector->singularize('remove' . $this->inflector->classify($fieldName));
                $item->{$methodName}($associatedObject);
                $modelManager->flush();
                return JsonResponse::create([], Response::HTTP_NO_CONTENT);
            }
        }
        throw new BadRequestHttpException("Bad Request");
    }
}
