<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use InvalidArgumentException;
use App\Service\MetadataHelper;
use App\Constants\CacheConstants;
use Doctrine\Inflector\InflectorFactory;
use App\Entity\ModelMetadata\ModelMetadata;
use Symfony\Component\HttpFoundation\Response;
use App\Entity\ModelMetadata\StagingModelMetadata;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

abstract class BaseController extends AbstractController
{
    protected $inflector;

    protected $resourceClassResolver;

    protected $containers;

    protected $cache;

    protected $metadataHelper;

    const NORMALIZER_METHOD = 'jsonld';
    const SELECTIVE_FIELDS = '__selectFields';
    const API_PREFIX = '/api/3/';
    const RESOURCE_SECURITY = 'security';

    public function __construct(ResourceClassResolverInterface $resourceClassResolver, ContainerInterface $containers, 
        TagAwareCacheInterface $cache, MetadataHelper $metadataHelper)
    {
        $this->inflector = InflectorFactory::create()->build();
        $this->resourceClassResolver = $resourceClassResolver;
        $this->containers = $containers;
        $this->cache = $cache;
        $this->metadataHelper = $metadataHelper;
    }

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    public function getClassNameByType($type)
    {
        //TODO: get this from metadata helper
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(ModelMetadata::class);
        $queryRepository = $queryManager->getRepository(ModelMetadata::class);
        try {
            return $queryRepository->findOneBy(array('type' => $type));
        } catch (InvalidArgumentException | \Exception $e) {
            error_log($e->getMessage());
            throw new NotFoundHttpException('No module for type found');
        }
    }

    public function getResourceFromClassName($resourceShortName)
    {
        return $this->metadataHelper->getResourceFromClassName($resourceShortName);
    }

    public function getSuccessResponse($data, $resource = null, $status = Response::HTTP_OK)
    {
        $normalized_data = $this->get('serializer')->normalize(
            $data,
            self::NORMALIZER_METHOD,
            ['resource_class' => $resource]
        );
        $response = new JsonResponse(
            $normalized_data,
            $status
        );
        return $response;
    }
}
