<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Security\Voter\FsrVoter;
use App\Service\BulkRequestService;
use Symfony\Component\HttpFoundation\Request;
use ApiPlatform\Core\Api\IriConverterInterface;
use App\Providers\PermissionsDictionaryBuilder;
use Symfony\Component\Serializer\SerializerInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use App\Service\MetadataHelper;
use App\Service\UtilityService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class BulkUpdateController extends BaseController
{
    /** @var  FsrVoter */
    private $fsrVoter;

    protected $bulkRequestService;

    protected $metadataHelper;

    public function __construct(FsrVoter $fsrVoter, ResourceClassResolverInterface $resourceClassResolver,
        ContainerInterface $containers, IriConverterInterface $iriConverter,
        SerializerInterface $serializer, BulkRequestService $bulkRequestService, MetadataHelper $metadataHelper, TagAwareCacheInterface $cache) {
        $this->fsrVoter = $fsrVoter;
        $this->iriConverter = $iriConverter;
        $this->serializer = $serializer;
        $this->bulkRequestService = $bulkRequestService;
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    public function bulkUpdateAction(Request $request, $moduletype)
    {
        $resourceClass = $this->getResourceFromClassName($moduletype);
        $module = $this->metadataHelper->getModuleFromClassName($resourceClass);
        if (!($this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.' . $module))) {
            throw new AccessDeniedException();
        }
        $resourceClass = $this->getResourceFromClassName($moduletype);

        // TODO: validate if payload is an array or single object
        $objectData = json_decode($request->getContent());
        $headers = $request->headers->all();
        $queryParams = $request->query->all();
        return $this->bulkRequestService->prepareUpdateDeleteSubRequest($resourceClass, $objectData, '/api/3/' . $moduletype, $headers, $queryParams, 'PUT');
    }
}
