<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use Ramsey\Uuid\Uuid;
use App\Entity\Core\File;
use App\Entity\Core\Image;
use App\Entity\Core\PgFile;
use App\Entity\Actors\Avatar;
use App\Entity\Custom\Person;
use InvalidArgumentException;
use App\Service\MetadataHelper;
use App\Service\UtilityService;
use App\Security\Voter\FsrVoter;
use App\Entity\Core\AbstractFile;
use App\Controller\BaseController;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use App\Providers\PermissionsDictionaryBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use ApiPlatform\Core\DataProvider\ItemDataProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class FilesController extends BaseController
{

    const WIDTH = 65;
    const HEIGHT = 65;

    protected $fsrVoter;
    protected $itemDataProvider;
    private $entityManager;
    private $utilityService;

    public function __construct(
        FsrVoter $fsrVoter,
        ContainerInterface $containers,
        EntityManagerInterface $entityManager,
        ItemDataProviderInterface $itemDataProvider,
        UtilityService $utilityService,
        MetadataHelper $metadataHelper,
        ResourceClassResolverInterface $resourceClassResolver,
        TagAwareCacheInterface $cache
    ) {
        $this->fsrVoter = $fsrVoter;
        $this->itemDataProvider = $itemDataProvider;
        $this->entityManager = $entityManager;
        $this->utilityService = $utilityService;

        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    public function getFile(Request $request, $id)
    {
        if ($request->attributes->get('_api_item_operation_name') == 'get_fsr_avatar') {
            $resource =  Avatar::class;
        } else {
            $resource = $request->attributes->get('_api_resource_class') ?? File::class;
        }

        $accept = $request->headers->get('Accept', 'application/json');
        if ($id == 'current') {
            $id = $this->getUser()->getUuid();
        }
        $request->attributes->set('id', $id);
        /** @var AbstractFile $file */
        $file = $this->itemDataProvider->getItem($resource, ["uuid" => $id]);
        if (is_null($file)) {
            return new Response(null, Response::HTTP_NO_CONTENT);
        }
        if (strpos($accept, 'application/json') !== false) {
            if ($resource != Avatar::class && !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . 'files')) {
                throw new AccessDeniedException();
            }
            $normalized_data = $this->get('serializer')->normalize($file, self::NORMALIZER_METHOD, ['resource_class' => $resource]);
            if ($request->attributes->get('_api_item_operation_name') == 'get_fsr_image' || $request->attributes->get('_api_item_operation_name') == 'get_fsr_avatar') {
                $container = $this->containers->get('doctrine');
                $EntityManager = $container->getManagerForClass(PgFile::class);
                $EntityRepository = $EntityManager->getRepository(PgFile::class);
                $explodeId = explode('/', $file->getFile());
                $fid = end($explodeId);
                try {
                    $FileRecord = $EntityRepository->findOneBy(['uuid' => $fid]);
                } catch (InvalidArgumentException | \Exception $e) {
                    error_log($e->getMessage());
                    throw new NotFoundHttpException(sprintf('File with %s not found', $fid));
                }
                $normalized_data['file'] = [];
                $normalized_data['file']['data'] = stream_get_contents($FileRecord->getFile());
            }
            $response = new JsonResponse(
                $normalized_data,
                Response::HTTP_OK
            );
            return $response;
        } else {
            if ($file->getSize() > 0) {
                $container = $this->containers->get('doctrine');
                $EntityManager = $container->getManagerForClass(PgFile::class);
                $EntityRepository = $EntityManager->getRepository(PgFile::class);
                $explodeId = explode('/', $file->getFile());
                $fid = end($explodeId);
                try {
                    $FileRecord = $EntityRepository->findOneBy(['uuid' => $fid]);
                } catch (InvalidArgumentException | \Exception $e) {
                    error_log($e->getMessage());
                    throw new NotFoundHttpException(sprintf('File with %s not found', $fid));
                }
                $response = new Response();
                $response->headers->set('Content-Type', $file->getMimeType());
                $response->headers->set('Content-Disposition', 'attachment; filename="' . $file->getFilename() . '"');
                $response->headers->set('filename', $file->getFilename());
                $response->headers->set('Content-Length', $file->getSize());
                $response->setContent(base64_decode(stream_get_contents($FileRecord->getFile())));
                return $response;
            } else {
                return new Response(null, Response::HTTP_NO_CONTENT);
            }
        }
    }
    public function createFile(Request $request)
    {
        $fileUuid = $request->get('uuid') ?? Uuid::uuid4()->toString();
        $container = $this->containers->get('doctrine');
        if ($request->attributes->get('_api_item_operation_name') == 'post_fsr_image') {
            $resource =  Image::class;
        } elseif ($request->attributes->get('_api_item_operation_name') == 'post_fsr_current_avatar') {
            $resource =  Avatar::class;
            $fileUuid = $this->getUser()->getUuid();
            $file = $this->itemDataProvider->getItem($resource, ["uuid" => $fileUuid]);
            if ($file) {
                $upload = $request->files->get('file');
                if (!$upload->isValid()) {
                    throw new BadRequestHttpException($upload->getErrorMessage());
                }
                $fstrm = fopen($upload->getPathname(), 'rb');
                $strm = stream_get_contents($fstrm);
                $file->setFilename($upload->getClientOriginalName());
                $file->setMimeType($upload->getClientMimeType());
                $file->setSize($upload->getSize());
                $entityManager = $this->containers->get('doctrine')->getManagerForClass(PgFile::class);
                $file_explode = explode('/', $file->getFile());
                $pgFile = $entityManager->find(PgFile::class, end($file_explode));
                $pgFile->setFile(base64_encode($strm));
                $entityManager->persist($pgFile);
                $this->entityManager->persist($file);
                $this->entityManager->flush();
                $normalized_data = $this->get('serializer')->normalize(
                    $file,
                    self::NORMALIZER_METHOD,
                    ['resource_class' => $resource]
                );
                $normalized_data['file'] = [];
                $normalized_data['file']['data'] = $pgFile->getFile();
                $response = new JsonResponse(
                    $normalized_data,
                    Response::HTTP_CREATED
                );
                return $response;
            }
        } else {
            $resource =  File::class;
        }


        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_CREATE . '.' . 'files')) {
            throw new AccessDeniedException();
        }

        if (!$request->files->has('file')) {
            $request_body = json_decode($request->getContent(), true);
            if ($request_body['content'] && $request_body['filename']) {
                $tempPath = "/tmp/widget/" . $request_body['filename'];
                $tempFileSystem = new Filesystem();
                $tempFileSystem->dumpFile($tempPath, $request_body['content']);
                /** @var UploadedFile $upload */
                $uploadedFile = new uploadedFile($tempPath, $request_body['filename']);
                $upload = $request->files->get('file', $uploadedFile);
                $request_body['metadata'] ? $request->request->set('metadata', json_encode($request_body['metadata'])) : $request->request->set('metadata', json_encode([]));
            }
        } else {
            /** @var UploadedFile $upload */
            $upload = $request->files->get('file');
            if (!$upload->isValid()) {
                throw new BadRequestHttpException($upload->getErrorMessage());
            }
            if ($resource == Image::class) {
                $allowed_mime_type = $this->containers->getParameter('allowed_image_mime_type');
                if (!in_array($upload->getMimeType(), $allowed_mime_type)) {
                    throw new BadRequestHttpException("Invalid file specified");
                }
            }
        }
        $file = $this->itemDataProvider->getItem($resource, ["uuid" => $fileUuid]);
        if (!$file) {
            /** @var AbstractFile $file */
            $file = new $resource();
        }
        $file->setFilename($upload->getClientOriginalName());
        $file->setMimeType($upload->getClientMimeType());
        $file->setSize($upload->getSize());
        $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));
        $file->setUploadDate($timestamp);
        $file->setMetadata(json_decode($request->request->get('metadata', "{}"), true));

        $pgFile = new PgFile();

        $EntityManager = $container->getManagerForClass(PgFile::class);
        /** @var AbstractFile $file */
        $strm = fopen($upload->getPathname(), 'rb');
        $pgFile->setFile(base64_encode(stream_get_contents($strm)));
        $EntityManager->persist($pgFile);
        $EntityManager->flush();
        if (substr_count($file->getMimeType(), 'image/')) {
            list($wid, $ht) = getimagesize($upload->getPathname());
            $thumbnail = "data:image/jpeg;base64," . base64_encode($this->getImageThumbnail($pgFile->getFile(), $wid, $ht));
            $file->setThumbnail($thumbnail);
        }
        $file->setFile("/api/3/pgfile/" . $pgFile->getUuid());
        $file->setUuid($fileUuid);
        $file->setId($fileUuid);
        $this->entityManager->persist($file);
        $this->entityManager->flush();
        $normalized_data = $this->get('serializer')->normalize(
            $file,
            self::NORMALIZER_METHOD,
            ['resource_class' => $resource]
        );
        if ($request->attributes->get('_api_item_operation_name') == 'post_fsr_image' || $request->attributes->get('_api_item_operation_name') == 'post_fsr_current_avatar') {

            $normalized_data['file'] = [];
            $normalized_data['file']['data'] = $pgFile->getFile();
        }
        $response = new JsonResponse(
            $normalized_data,
            Response::HTTP_CREATED
        );
        return $response;
    }
    public function updateFile(Request $request, $id)
    {
        if ($request->attributes->get('_api_item_operation_name') == 'post_fsr_image') {
            $resource =  Image::class;
        } elseif ($request->attributes->get('_api_item_operation_name') == 'post_fsr_current_avatar') {
            $resource =  Avatar::class;
            $id = $this->getUser()->getUuid();
        } else {
            $resource =  File::class;
        }
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.' . 'files')) {
            throw new AccessDeniedException();
        }
        $file = $this->itemDataProvider->getItem($resource, ["uuid" => $id]);
        if (!$file) {
            throw new NotFoundHttpException("Record with Provided UUID does not exists");
        }
        if ($resource != Image::class && $resource != Avatar::class) {
            $this->utilityService->ValidateOwnerShip($request, $file, $resource);
        }
        if (!$request->files->has('file')) {
            $request_body = json_decode($request->getContent(), true);
            $strm = $request_body['content'];
            $file->setSize(strlen($request_body['content']));
            $filename = $request_body['filename'] ?? $file->getFilename();
            $tempPath = "/tmp/widget/" . $filename;
            $tempFileSystem = new Filesystem();
            $tempFileSystem->dumpFile($tempPath, $request_body['content']);
            /** @var UploadedFile $upload */
            $uploadedFile = new uploadedFile($tempPath, $filename);
            $upload = $request->files->get('file', $uploadedFile);
            if ($request_body['metadata'] ?? false) {
                $request->request->set('metadata', json_encode($request_body['metadata']));
            } elseif ($file->getMetadata()) {
                $request->request->set('metadata', $file->getMetadata());
            } else {
                $request->request->set('metadata', $file->getMetadata(json_encode([])));
            }
        } else {
            /** @var UploadedFile $upload */
            $upload = $request->files->get('file');
            if (!$upload->isValid()) {
                throw new BadRequestHttpException($upload->getErrorMessage());
            }
            $fstrm = fopen($upload->getPathname(), 'rb');
            $strm = stream_get_contents($fstrm);


            $file->setFilename($upload->getClientOriginalName());
            $file->setMimeType($upload->getClientMimeType());
            $file->setSize($upload->getSize());
            $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
            $file->setUploadDate($timestamp);
            $file->setMetadata(json_decode($request->request->get('metadata', "{}"), true));
        }
        $entityManager = $this->containers->get('doctrine')->getManagerForClass(PgFile::class);
        $file_explode = explode('/', $file->getFile());
        $pgFile = $entityManager->find(PgFile::class, end($file_explode));
        $pgFile->setFile(base64_encode($strm));
        $entityManager->persist($pgFile);
        $entityManager->flush();

        $denormalizedObject = $this->get('serializer')->deserialize(
            $request->getContent(),
            $resource,
            'jsonld',
            ['object_to_populate' => $file, 'fetch_data' => false]
        );
        $entityManager = $this->containers->get('doctrine')->getManagerForClass(File::class);
        $this->entityManager->persist($denormalizedObject);
        $this->entityManager->flush();
        return $this->getSuccessResponse($denormalizedObject);
    }
    public function deleteFile(Request $request, $id)
    {
        $checkFilePermission = true;
        if ($request->attributes->get('_api_item_operation_name') == 'delete_fsr_image') {
            $resource =  Image::class;
        } elseif ($request->attributes->get('_api_item_operation_name') == 'delete_fsr_avatar') {
            $actorObj = $this->entityManager->getRepository(Person::class)->findOneBy(['avatar'=> '/api/3/avatars/' . $id]);
            $id = $this->getUser()->getUuid();
            if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_DELETE . '.' . 'security')) {
                if ($actorObj && $actorObj->getUuid() != $id) {
                    throw new AccessDeniedException();
                }
            }
            if ($actorObj){
                $actorObj->setAvatar(null);
            }
            $checkFilePermission = false;
            $resource =  Avatar::class;
            
        } else {
            $resource =  File::class;
        }
        if ($checkFilePermission && !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_DELETE . '.' . 'files')) {
            throw new AccessDeniedException();
        }
        $file = $this->itemDataProvider->getItem($resource, ["uuid" => $id]);
        if (!$file) {
            throw new NotFoundHttpException("Record with Provided UUID does not exists");
        }
        if ($resource != Image::class && $resource != Avatar::class) {
            $this->utilityService->ValidateOwnerShip($request, $file, $resource);
        }
        $this->removeFiles($file);
        return new Response(null, Response::HTTP_NO_CONTENT);
    }

    protected function getImageThumbnail($imageString, $wid, $ht)
    {
        $r = $wid / $ht; {
            if (self::WIDTH / self::HEIGHT > $r) {
                $new_width = self::HEIGHT * $r;
                $new_height = self::HEIGHT;
            } else {
                $new_height = self::WIDTH / $r;
                $new_width = self::WIDTH;
            }
        }
        $source = imagecreatefromstring(base64_decode($imageString));
        $dst = imagecreatetruecolor($new_width, $new_height);
        imagecopyresampled($dst, $source, 0, 0, 0, 0, $new_width, $new_height, $wid, $ht);
        ob_start();
        imagejpeg($dst, null, 100);
        return ob_get_clean();
    }

    public function saveFiles($fileObj)
    {
        $this->entityManager->persist($fileObj);
        $this->entityManager->flush();
        return $fileObj;
    }

    public function removePgFile($fileObj)
    {
        $pgFileRepo = $this->entityManager->getRepository(PgFile::class);
        if ($fileObj->getFile()) {
            $file_explode = explode('/', $fileObj->getFile());
            $pgFileId = end($file_explode);
            $pgFile = $pgFileRepo->findOneBy(array("uuid" => $pgFileId));
            $this->entityManager->remove($pgFile);
            $this->entityManager->flush();
        }
    }

    public function removeFiles($fileObj)
    {
        $this->removePgFile($fileObj);
        $this->entityManager->remove($fileObj);
        $this->entityManager->flush();
        return $fileObj;
    }
}
