<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use ApiPlatform\Core\DataProvider\ItemDataProviderInterface;
use App\Entity\ViewTemplate\GenericTemplate;
use App\Entity\ViewTemplate\UserDefaultTemplate;
use App\Providers\QueryProvider;
use App\Security\Voter\FsrVoter;
use App\Service\GenericTemplateService;
use stdClass;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class GenericTemplateController extends AbstractController
{

    protected $genericTemplateService;

    protected $fsrVoter;

    protected $chainItemDataProvider;

    protected $queryProvider;

    public function __construct(GenericTemplateService $genericTemplateService, FsrVoter $fsrVoter, ItemDataProviderInterface $chainItemDataProvider, QueryProvider $queryProvider)
    {
        $this->genericTemplateService = $genericTemplateService;
        $this->fsrVoter = $fsrVoter;
        $this->chainItemDataProvider = $chainItemDataProvider;
        $this->queryProvider = $queryProvider;
    }

    protected $allowedType = ['dashboard', 'reporting'];
    const RELATIONSHIP_KEY = "\$relationships";
    const ENABLED = 'enabled';

    /**
     * @param $type String
     */
    private function isAllowedType($type)
    {
        if (!in_array($type, $this->allowedType)) {
            throw new BadRequestHttpException("Invalid Type.");
        }
    }

    /**
     * @param array genericTemplate
     * @return array template
     */
    private function filterRoles($genericTemplate)
    {
        $genericTemplateProvider = $this->genericTemplateService;
        if (array_key_exists('roles', $genericTemplate)) {
            $validRoleIds = $genericTemplateProvider->getAllRoles();
            $assignedRoles = $genericTemplate['roles'];
            $genericTemplate['roles'] = array_values(array_intersect($assignedRoles, $validRoleIds));
        }
        return $genericTemplate;
    }

    /**
     * @param String $permissionType
     */
    private function checkAuthorized($permissionType)
    {
        if (!$this->fsrVoter->isGranted($permissionType . ".security")) {
            return false;
        }
        return true;
    }

    public function createDefaultUserTemplateAction(Request $request, $type, $id)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->genericTemplateService;
        $resource = GenericTemplate::class;
        $genericTemplate = $this->chainItemDataProvider->getItem($resource, ["uuid" => $id]);
        if (!$genericTemplate) {
            throw new NotFoundHttpException("Template with given id does not exists");
        }
        $savedTemplate = $genericTemplateProvider->saveDefaultUserTemplate($type, $id, $genericTemplate);
        if ($savedTemplate) {
            return JsonResponse::create('Success');
        } else {
            return new Response(null, Response::HTTP_FORBIDDEN);
        }
    }

    public function getDefaultUserTemplateAction(Request $request, $type)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->genericTemplateService;
        $user = $genericTemplateProvider->getCurrentUser()->getUuid();
        $entityManager = $this->get('doctrine');
        $entityRepo = $entityManager->getRepository(UserDefaultTemplate::class);
        $defaultTemplate = $entityRepo->findBy(["userId" => $user]);
        if (count($defaultTemplate)) {
            return new JsonResponse($this->get('serializer')->normalize($defaultTemplate, 'jsonld'), Response::HTTP_OK);
        } else {
            return new JsonResponse([], Response::HTTP_OK);
        }
    }

    public function deleteDefaultUserTemplateAction($type)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->genericTemplateService;
        $user = $genericTemplateProvider->getCurrentUser()->getUuid();
        $templateEntityManager = $this->get('doctrine');
        $templateRepo = $templateEntityManager->getRepository(UserDefaultTemplate::class);
        $template = $templateRepo->findOneBy(array("userId" => $user));
        if (!is_null($template)) {
            $genericTemplateProvider->deleteTemplate($template);
            return JsonResponse::create('Success');
        } else {
            return new Response(null, Response::HTTP_NOT_FOUND);
        }
    }

    public function resetTemplateAction(Request $request, $type, $id)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->genericTemplateService;
        $resource = GenericTemplate::class;
        $genericTemplate = $this->chainItemDataProvider->getItem($resource, ["uuid" => $id]);
        if (!$genericTemplate) {
            throw new NotFoundHttpException("Template with Provided UUID does not exists");
        }
        $parentTemplateId = $genericTemplate->getParentTemplateId();
        if (!empty($parentTemplateId) && !is_null($parentTemplateId)) {
            $parentDashBoard = $this->chainItemDataProvider->getItem($resource, ["uuid" => $parentTemplateId]);
            if (!$parentDashBoard) {
                throw new NotFoundHttpException("Parent Template ID provided does not exists");
            }
            $genericTemplate->setConfig($parentDashBoard->getConfig());
            $genericTemplateProvider->saveTemplate($genericTemplate);
        }
        return JsonResponse::create('Success');
    }
}
