<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use InvalidArgumentException;
use App\Security\Voter\FsrVoter;
use Symfony\Component\Yaml\Yaml;
use App\Entity\Core\EmailTemplate;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use ApiPlatform\Core\DataProvider\ItemDataProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Finder\Exception\AccessDeniedException;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class ManualInputController extends AbstractController
{

    protected $fsrVoter;

    protected $containers;

    protected $tokenStorage;

    protected $itemDataProvider;

    protected $em;

    public function __construct(FsrVoter $fsrVoter, ContainerInterface $containers, TokenStorageInterface $tokenStorage, ItemDataProviderInterface $itemDataProvider, EntityManagerInterface $entityManager)
    {
        $this->fsrVoter = $fsrVoter;
        $this->containers = $containers;
        $this->tokenStorage = $tokenStorage;
        $this->itemDataProvider = $itemDataProvider;
        $this->em = $entityManager;
    }

    private function getDasClient()
    {
        return $this->containers->get("app.proxy.client.das");
    }

    private function getIntegrationClient()
    {
        return $this->containers->get("app.proxy.client.integrations");
    }

    public function sendMail($templateId, $dataToReplace, $mailDetails, $integrationClient, $defaultMessage)
    {
        $resource = EmailTemplate::class;
        $email_template = [];
        $headers = array(
            "Content-type" => "application/json",
        );
        try {
            $object = $this->itemDataProvider->getItem($resource, ["uuid" => $templateId]);
        } catch (InvalidArgumentException| \Exception $error) {
            $object = false;
        }

        if ($object) {
            /** @var Serializer $serializer */
            $email_template = $this->get('serializer')->normalize($object, 'jsonld');
        } else {
            $email_options = Yaml::parse(file_get_contents('/opt/cyops/configs/cyops-api/mailtemplate.yml'));
            $email_template = $email_options[$defaultMessage];
        }

        foreach ($dataToReplace as $key => $value) {
            $email_template['content'] = str_replace($key, $value, $email_template['content']);
        }

        $smtpObject = '{
                            "connector": "smtp",
                            "operation": "send_richtext_email",
                            "params": {
                                "cc": "",
                                "bcc": "",
                                "content_type":"text/html"
                            }
                        }';

        $jsonDecodedSmtp = json_decode($smtpObject, true);
        $jsonDecodedSmtp['params']['subject'] = $email_template['subject'];
        $jsonDecodedSmtp['params']['content'] = $email_template['content'];
        $jsonDecodedSmtp['params']['to'] = $mailDetails['to'];
        $smtpUri = $this->containers->getParameter('integrations.uri');
        $smtpUri = $smtpUri . '/execute/?format=json';
        $smtpRequest = new GuzzleRequest('post', $smtpUri, $headers, json_encode($jsonDecodedSmtp));
        try {
            $integrationClient->send($smtpRequest);
        } catch (BadRequestHttpException $e) {
            $errMsg = json_decode($e->getResponse()->getBody()->read(2048), true);
            error_log($errMsg['message']);
            return [
                'success' => false,
                'message' => $errMsg['message']
            ];
        }
        catch (\Exception $exception) {
            $errMsg = json_decode($exception->getResponse()->getBody()->read(2048), true);
            error_log($errMsg['message']);
            return [
                'success' => false,
                'message' => $errMsg['message']
            ];
        }
        return [
            'success' => true
        ];
    }

    public function sendManualInputEmailsAction(Request $request)
    {
        $this->fsrVoter->denyUnlessAccesssGranted("read.security");
        if ($request->getHost() !== 'localhost') {
            throw new AccessDeniedException();
        }
        $dasUri = $this->containers->getParameter('das.uri');
        $dasClient = $this->getDasClient();
        $postData = json_decode($request->getContent(), true);
        foreach ($postData['recipients'] as $emailAddress) {
            $retrieveWfInputIRI = sprintf('/api/wf/api/manual-wf-input/%s/retrieve_wfinput/', $postData['inputId']);
            $wfinputResumeIRI = sprintf('/api/wf/api/workflows/%s/wfinput_resume/', $postData['workflowId']);
            $objectData = [
                "action" => "generate_restricted_token",
                "data" => [
                    "email" => $emailAddress,
                    "expiry" => $postData['expiry'],
                    "allowed_iris" => [
                        ['iri' => $retrieveWfInputIRI, 'method' => 'POST'],
                        ['iri' => $wfinputResumeIRI, 'method' => 'POST'],
                        ['iri' => '/api/3/logout', 'method' => 'POST']
                    ],
                    "invalidate_iri" => ['iri' => $wfinputResumeIRI, 'method' => 'POST']
                ]
            ];
            $headers = array(
                "Content-type" => "application/json",
            );
            $dasRequests = new GuzzleRequest('post', $dasUri . '/token', $headers, json_encode($objectData));
            try {
                $dasResponse = $dasClient->send($dasRequests);
                $dasResponseData = $dasResponse->getBody()->getContents();
                $jsonDecodeResponse = json_decode($dasResponseData, true);
            } catch (BadRequestHttpException| \Exception $error) {
                error_log($error->getMessage());
                $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
                $errMsg['message'] = $errVar['Error'];
                $errCode = $error->getCode();
                return JsonResponse::create($errMsg, $errCode);
            }
            $system_host = $postData['hostname'];
            $integrationClient = $this->getIntegrationClient();
            $dataToReplace = [];
            $dataToReplace['{{url}}'] = sprintf('%s://%s/input?inputId=%s&token=%s', $request->getScheme(), $system_host, $postData['inputId'], $jsonDecodeResponse['token']);
            $mailDetails = [];
            $mailDetails['to'] = $emailAddress;
            $defaultMessage = 'manualinputemail';
            $mailResponse = $this->sendMail('e736eb31-bc42-4e8c-b9e2-d99ccdd4ee8b', $dataToReplace, $mailDetails, $integrationClient, $defaultMessage);
            if (!$mailResponse['success']) {
                $errorResponse['message'] = sprintf("Attempted to send a manual input request, but the email was not sent due to a SMTP connector failure:\n%s", $mailResponse['message']);
                return JsonResponse::create($errorResponse, 400);
            }
        }
        $successMsg['message'] = "Manual input request sent to user";
        return JsonResponse::create($successMsg);
    }
}
