<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Entity\Actors\Actor;
use App\Entity\Custom\Person;
use App\Security\Voter\FsrVoter;
use Symfony\Component\Yaml\Yaml;
use App\Entity\Settings\SystemSettings;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use App\EventSubscriber\MqMessagebroadcastSubscriber;
use ApiPlatform\Core\DataProvider\ItemDataProviderInterface;
use App\Service\UtilityService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class PasswordResetController extends AbstractController
{

    protected $fsrVoter;
    protected $containers;
    protected $tokenStorage;
    protected $itemDataProvider;
    protected $em;
    private $utilityService;

    public function __construct(
        FsrVoter $fsrVoter,
        ContainerInterface $containers,
        TokenStorageInterface $tokenStorage,
        ItemDataProviderInterface $itemDataProvider,
        EntityManagerInterface $entityManager,
        UtilityService $utilityService)
    {
        $this->fsrVoter = $fsrVoter;
        $this->containers = $containers;
        $this->tokenStorage = $tokenStorage;
        $this->itemDataProvider = $itemDataProvider;
        $this->em = $entityManager;
        $this->utilityService = $utilityService;
    }

    private function getDasClient()
    {
        return $this->containers->get("app.proxy.client.das");
    }

    private function getIntegrationClient()
    {
        return $this->containers->get("app.proxy.client.integrations");
    }

    private function getProxyClient()
    {
        return $this->containers->get("app.proxy.client.workflow");
    }

    private function getCurrentUser()
    {
        return $this->tokenStorage->getToken()->getUser();
    }

    public function checkAuthorization($method)
    {
        if ($method === 'POST') {
            if (!$this->fsrVoter->isGranted('create.security') && !$this->fsrVoter->isGranted('update:security')) {
                throw new AccessDeniedException();
            }
        }
    }

    

    public function resetPasswordAction(Request $request)
    {

        $this->checkAuthorization($request->getMethod());
        $dasUri = $this->containers->getParameter('das.uri');
        $dasClient = $this->getDasClient();
        $objectData = json_decode($request->getContent(), true);
        $headers = array(
            "Content-type" => "application/json",
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/password', $headers, $request->getContent());

        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
        } catch (BadRequestHttpException | \Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }

        if ($resCode < 300) {
            if ($objectData['sendEmail']) {
                $integrationClient = $this->getIntegrationClient();
                $dataToReplace = [];
                $mailDetails = [];
                $defaultMessage = 'adminresetpassword';
                $resource = Person::class;
                $id = $objectData['currentUUID'];
                $object = $this->itemDataProvider->getItem($resource, ["uuid" => $id]);
                $item = $this->get('serializer')->normalize($object, 'jsonld', []);
                $title = "Password for " . $item['firstname'] . " " . $item['lastname'] . "[" . $objectData['loginId'] . "] Changed";
                $mailDetails['to'] = $item['email'];
                $updateObj = [];
                $updateObj['@type'] = $item['@type'];
                $updateObj['@id'] = $item['@id'];
                $dataToReplace['{{vars.userId}}'] = $objectData['loginId'];
                $body = $this->prepareChangeData($updateObj, (object)null, 'Event', [], $title);
                $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
                $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode([$body]), MqMessagebroadcastSubscriber::DEFALUT_KEY);
                $this->utilityService->sendMail('2218f5b9-0b17-4f21-91c9-5404696d1908', $dataToReplace, $mailDetails, $integrationClient, $defaultMessage);
            }
            return JsonResponse::create("Password Changed sucessfully");
        }
    }

    public function changePasswordAction(Request $request)
    {

        $dasUri = $this->containers->getParameter('das.uri');
        $dasClient = $this->getDasClient();
        $userId = $this->getCurrentUser()->getUserId();
        $userUuid = $this->getCurrentUser()->getUuid();
        $objectData = json_decode($request->getContent(), true);
        $objectData['user_id'] = $userId;
        $headers = array(
            "Content-type" => "application/json",
        );
        if ($userUuid === $objectData['uuid']) {
            $dasRequests = new GuzzleRequest('put', $dasUri . '/password', $headers, $request->getContent());
            try {
                $dasResponse = $dasClient->send($dasRequests);
                $resCode = $dasResponse->getStatusCode();
            } catch (BadRequestHttpException | \Exception $error) {
                error_log($error->getMessage());
                $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
                $errMsg['message'] = $errVar['Error'];
                $errCode = $error->getCode();
                return JsonResponse::create($errMsg, $errCode);
            }

            if ($resCode < 300) {
                $sealabClient = $this->getProxyClient();
                $dataToReplace = [];
                $mailDetails = [];
                $mailDetails['to'] = $this->getCurrentUser()->getEmail();
                $defaultMessage = 'changepassword';
                $dataToReplace['{{vars.userId}}'] = $userId;
                $mailResponse = $this->utilityService->sendMail('e71ba2df-e4ed-4823-9c28-b86423e9a7eb', $dataToReplace, $mailDetails, $sealabClient, $defaultMessage);
                if ($mailResponse === true) {
                    $sucessMsg['message'] = "Password Changed successfully";
                    return JsonResponse::create($sucessMsg);
                } else {
                    $sucessMsg['message'] = "Password Changed successfully, mail not sent due to smtp connector failure";
                    return JsonResponse::create($sucessMsg);
                }
            }
        } else {
            error_log("user Token mismatch");
            return JsonResponse::create("user Token mismatch", 400);
        }
    }

    protected function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }

    protected function prepareChangeData(
        $updatedObject,
        $previousObject,
        $method,
        $changeData = [],
        $title = null
    ) {
        $delta_data = [];
        $delta_data['oldData'] = $previousObject;
        $delta_data['newData'] = $updatedObject;
        $delta_data['changeData'] = $changeData;
        $userName = $this->extractUserName();
        $userId = $this->getUser()->getUuid();
        return [
            MqMessagebroadcastSubscriber::DELTA_DATA => $delta_data,
            MqMessagebroadcastSubscriber::HTTP_METHOD => $method,
            MqMessagebroadcastSubscriber::REMOTE_ADDR => $_SERVER['HTTP_X_FORWARDED_FOR'] ?? $_SERVER['REMOTE_ADDR'],
            MqMessagebroadcastSubscriber::WEB_SOCKET_SESSION_ID => isset($_SERVER['HTTP_WEBSOCKET_SESSIONID']) ? $_SERVER['HTTP_WEBSOCKET_SESSIONID'] : null,
            MqMessagebroadcastSubscriber::COMPONENT => 'crudhub',
            MqMessagebroadcastSubscriber::USER_UUID => $userId,
            MqMessagebroadcastSubscriber::TRANSACTION_DATE => round(microtime(true) * 1000),
            MqMessagebroadcastSubscriber::USER => $userName,
            MqMessagebroadcastSubscriber::TITLE => $title
        ];
    }
}
