<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Providers\PermissionsDictionaryProvider;
use App\Security\Voter\FsrVoter;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class PermissionsController extends AbstractController
{

    /** @var PermissionsDictionaryProvider */
    protected $permissionsDictionaryProvider;

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    protected $fsrVoter;

    /**
     * PermissionResolver constructor.
     * @param PermissionsDictionaryProvider $permissionsDictionaryProvider
     */
    public function __construct(PermissionsDictionaryProvider $permissionsDictionaryProvider, TokenStorageInterface $tokenStorage, FsrVoter $fsrVoter)
    {
        $this->permissionsDictionaryProvider = $permissionsDictionaryProvider;
        $this->tokenStorage = $tokenStorage;
        $this->fsrVoter = $fsrVoter;
    }

    public function listAction()
    {
        $actor = $this->tokenStorage->getToken()->getUser();
        $permissionsDictionary = $this->permissionsDictionaryProvider->getDataForActor($actor);

        return JsonResponse::create($permissionsDictionary);
    }

    /**
     * @param string $moduleName
     * @param string $action
     * @return JsonResponse
     */
    public function getAction($module, $action)
    {

        if (!$this->fsrVoter->isGranted($this->buildPermission($module, $action))) {
            return JsonResponse::create(false);
        }
        return JsonResponse::create(true);
    }

    /**
     * @param string $moduleName
     * @param string $action
     * @return string
     */
    private function buildPermission($moduleName, $action)
    {
        return sprintf('%s_%s', $action, $moduleName, );
    }
}
