<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Security\Voter\FsrVoter;
use App\Providers\PermissionsDictionaryBuilder;
use Symfony\Component\HttpFoundation\Request;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use App\Service\MetadataHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class UserRoleTeamController extends BaseController
{
   /** @var FsrVoter */
    private $fsrVoter;
    /** @var SerializerInterface */
    private $serializer;

    public function __construct(FsrVoter $fsrVoter, SerializerInterface $serializer, ResourceClassResolverInterface $resourceClassResolver, 
        ContainerInterface $containers, TagAwareCacheInterface $cache, MetadataHelper $metadataHelper){
        $this->fsrVoter = $fsrVoter;
        $this->serializer = $serializer;
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }
   
    /**
     * @param string $moduleName
     * @param string $action
     * @return JsonResponse
     */
    public function getAction(Request $request)
    {
        $this->fsrVoter->denyUnlessAccesssGranted(
            PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . PermissionsDictionaryBuilder::MODULE_APPLICATION);
        $personClass = $this->getResourceFromClassName('people');
        $normalizedTeamData = $this->serializer->normalize(
            $this->getUser(), BaseController::NORMALIZER_METHOD, 
            [   'resource_class' => $personClass,
                'groups' => ['teams', 'roles', 'uuid'],
                'ignore_authorization' => true, 
                'ignore_field_authorization' => true,
                'ignore_relationship_authorization' => false,
                'item_operation_name' => 'get',
                'collection_operation_name' => 'get',
            ]);

        return JsonResponse::create($normalizedTeamData);
    }
}