<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Entity\ViewTemplate\UserViewTemplate;
use App\Service\ViewTemplateService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class ViewTemplateController extends AbstractController
{

    protected $viewTemplateService;

    public function __construct(ViewTemplateService $viewTemplateService)
    {
        $this->viewTemplateService = $viewTemplateService;
    }

    /**
     * @param $name
     * @return JsonResponse
     */
    public function getAction($name)
    {
        $serializer = $this->get('serializer');

        $viewTemplate = $this->findOrThrowNotFound($name);

        $normalizedViewTemplate = $serializer->normalize($viewTemplate, 'json');

        return JsonResponse::create($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return JsonResponse
     */
    public function setAction($name, Request $request)
    {
        $viewTemplateProvider = $this->viewTemplateService;
        $serializer = $this->get('serializer');

        /** @var UserViewTemplate $userViewTemplate */
        $userViewTemplate = $serializer->deserialize(
            $request->getContent(),
            UserViewTemplate::class,
            'jsonld',
            ['object_to_populate' => $this->findOrThrowNotFound($name)]
        );

        $viewTemplateProvider->saveViewTemplate($name, $userViewTemplate);

        $normalizedViewTemplate = $serializer->normalize($userViewTemplate, 'json');

        return JsonResponse::create($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return mixed
     */
    private function findOrThrowNotFound($name)
    {
        $viewTemplateProvider = $this->viewTemplateService;

        $viewTemplate = $viewTemplateProvider->getViewTemplate($name);

        return $viewTemplate;
    }
}
