<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Constants\Triggers;
use App\Entity\Actors\Actor;
use InvalidArgumentException;
use App\Service\MetadataHelper;
use App\Service\WorkflowHelper;
use App\Security\Voter\FsrVoter;
use App\Entity\Workflow\Workflow;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\HttpFoundation\Response;
use ApiPlatform\Core\Api\IriConverterInterface;
use App\Providers\PermissionsDictionaryBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use App\EventSubscriber\MqMessagebroadcastSubscriber;
use Symfony\Component\Serializer\SerializerInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class WorkflowTriggerController extends BaseController
{
    /** @var  SerializerInterface */
    private $serializer;
    /** @var  EntityManagerInterface */
    private $entityManager;
    /** @var WorklowHelper */
    protected $workflowHelper;
    /** @var  FsrVoter */
    private $fsrVoter;
    private $iriConverter;

    const RESOURCE_PARAM = 'resource';
    const ROUTE_PARAM = 'route';
    const DELTA_DATA = 'data';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const WEB_SOCKET_SESSION_ID = 'websocketId';
    const COMPONENT = 'component';
    const UUID = 'entityUuid';
    const USER_UUID = 'userId';
    const USER = 'user';
    const TRANSACTION_DATE = 'transactionDate';
    const PLAYBOOK_IRI = 'playbookIri';
    const PLAYBOOK_NAME = 'playbookName';

    public function __construct(
        FsrVoter $fsrVoter,
        SerializerInterface $serializer,
        EntityManagerInterface $entityManager,
        WorkflowHelper $workflowHelper,
        ResourceClassResolverInterface $resourceClassResolver,
        IriConverterInterface $iriConverter,
        ContainerInterface $containers,
        TagAwareCacheInterface $cache,
        MetadataHelper $metadataHelper
    ) {
        $this->fsrVoter = $fsrVoter;
        $this->serializer = $serializer;
        $this->entityManager = $entityManager;
        $this->workflowHelper = $workflowHelper;
        $this->iriConverter = $iriConverter;
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    public function postManualAction(Request $request, $workflowid)
    {
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE . '.' . Triggers::WORKFLOW_RESOURCE)) {
            throw new AccessDeniedException();
        }
        $audit_array = [];
        $body = json_decode($request->getContent(), true);
        $data = $this->extractDataFromRequest($body);
        if (array_key_exists("__uuid", $data)) {
            $workflowId = $data['__uuid'];
        } else {
            throw $this->createNotFoundException('Workflow UUID Not Found In Request');
        }

        $workflowObj = $this->fetch_workflow_data($workflowId);
        $trigger = Triggers::ACTION;
        $headerObj['X-TRIGGERTYPE'] = Triggers::ACTION;
        //Check authorization on action trigger

        if (isset($body['singleRecordExecution']) && $body['singleRecordExecution'] == true) {
            $task_ids = [];

            $clonedRecordData = $data;
            unset($clonedRecordData['records']);
            foreach ($data['records'] as $record) {
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
                $clonedRecordData['records'] = [$record];
                array_push($audit_array, $this->prepareChangeData(
                    $clonedRecordData['records'],
                    (object)null,
                    'Trigger',
                    $workflowObj['@id'],
                    $workflowObj['name']
                ));
                $requestBody = $this->workflowHelper->prepareWorkflowRequestBody($request, $workflowid, $clonedRecordData, null);
                $workflow_response = $this->workflowHelper->handleTrigger($trigger, $requestBody, $workflowObj, $headerObj);
                array_push($task_ids, json_decode($workflow_response)->task_id);
            }
            $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
            $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($audit_array), MqMessagebroadcastSubscriber::DEFALUT_KEY);
            return new JsonResponse([
                'task_ids' => $task_ids
            ]);
        } else {
            $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
            $requestBody = $this->workflowHelper->prepareWorkflowRequestBody($request, $workflowid, $data, null);
            $workflow_response = $this->workflowHelper->handleTrigger($trigger, $requestBody, $workflowObj, $headerObj);
            array_push($audit_array, $this->prepareChangeData(
                $data['records'],
                (object)null,
                'Trigger',
                $workflowObj['@id'],
                $workflowObj['name']
            ));
            $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
            $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($audit_array), MqMessagebroadcastSubscriber::DEFALUT_KEY);
            return JsonResponse::create(json_decode($workflow_response, true));
        }
    }

    public function noTriggerExecuteAction(Request $request, $workflowId)
    {
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE . '.' . Triggers::WORKFLOW_RESOURCE)) {
            throw new AccessDeniedException();
        }
        $body = json_decode($request->getContent(), true);
        $body['request']['method'] = $request->getMethod();
        $body['request']['uri'] = $request->getUri();
        $body['request']['baseUri'] = 'https://' . $request->getHost();
        $body['request']['currentUser'] = '/api/3/people/' . $this->getUser()->getUuid();
        $body['request']['headers'] = $this->getHeaders($request->headers->all());

        $normalized_data = $this->fetch_workflow_data($workflowId);
        $triggerStepIri = isset($normalized_data['triggerStep']) ? $normalized_data['triggerStep'] : null;

        foreach ($normalized_data['steps'] as &$normalizedStep) {
            if ($normalizedStep['@id'] === $triggerStepIri) {
                $normalizedStep['arguments'] = array_merge($normalizedStep['arguments'], $body);
                if (isset($normalizedStep['arguments']['inputVariables']) && count($normalizedStep['arguments']['inputVariables']) > 0) {
                    foreach ($normalizedStep['arguments']['inputVariables'] as $userInput) {
                        $normalizedStep['arguments']['request']['data'][$userInput['name']] = $normalizedStep['arguments'][$userInput['name']];
                    }
                }
                break;
            }
        }

        $token = $this->get('security.token_storage')->getToken();
        $headers = array(
            "Content-type" => "application/json",
        );
        $headers['Authorization'] = "Bearer " . $token->getCredentials();
        $headers['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
        $headers['X-TRIGGERTYPE'] = Triggers::NOTRIGGER;

        $body = $this->get('serializer')->encode(
            $normalized_data,
            'json'
        );
        $proxyClient = $this->containers->get("app.proxy.client.workflow");
        $workflowRequestUri = $proxyClient->getTargetUri() . $this->getParameter('workflow_endpoint');
        $workflowRequest = new GuzzleRequest('post', $workflowRequestUri, $headers, $body);
        try {
            $response = $proxyClient->send($workflowRequest);
            $statusCode = $response->getStatusCode();
            if ($statusCode >= 400) {
                $errMsg = "Failed to get response from Workflow log";
                return JsonResponse::create($errMsg, $statusCode);
            } else {
                $jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return JsonResponse::create($jsonDecodeResponse, $statusCode);
            }
        } catch (BadRequestHttpException $e) {
            $errMsg['message'] = $e->getMessage();
            $errCode = $e->getCode();
            error_log($errMsg['message']);
            return JsonResponse::create($errMsg, $errCode);
        } catch (\Exception $exception) {
            // TODO: match response when workflow api returns non OK response
            // 502 bad gateway is only handled currently
            $errMsg = $exception->getMessage();
            $errCode = $exception->getCode();
            return JsonResponse::create(array('message' => $errMsg), $errCode);
        }
    }

    public function apiAction(Request $request, $apiEndpoint)
    {
        $body = $request->getContent();
        $data = json_decode($body, true);
        if ($this->getUser() != null) {
            if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE . '.' . Triggers::WORKFLOW_RESOURCE)) {
                throw new AccessDeniedException();
            }
            if (method_exists($this->getUser(), "getfirstname")) {
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
            } else {
                $headerObj['X-RUNBYUSER'] = '/api/3/appliance/' . $this->getUser()->getUuid();
            }
        } else if (isset($_SERVER['PHP_AUTH_USER'])) {
            $loginId = $_SERVER['PHP_AUTH_USER'];
            $userId = $this->getUserId($loginId);
            if ($userId != null) {
                $userUuid = $this->getuuid($userId);
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $userUuid;
            } else {
                $headerObj['X-RUNBYUSER'] = "";
            }
        } else {
            $headerObj['X-RUNBYUSER'] = "";
        }
        $headerObj['X-TRIGGERTYPE'] = Triggers::API_CALL;
        $workflowRepository = $this->entityManager->getRepository(Workflow::class);
        $workflows = $workflowRepository->getWorkflowsForTriggerType(Triggers::API_CALL, null, null);
        if (empty($workflows)) {
            return new JsonResponse(array('message' => 'No active workflow matching the given route.'), Response::HTTP_NOT_FOUND);
        }
        $taskIds = array();
        foreach ($workflows as $workflow) {
            $triggerStep = $workflow->getTriggerStep();
            $arguments = $triggerStep->getArguments();

            # TODO: needs to be handled for API trigger
            $triggerParameter = self::ROUTE_PARAM;

            //Validate trigger step
            if (!array_key_exists($triggerParameter, $arguments)) {
                continue;
            }
            if ($arguments[$triggerParameter] != $apiEndpoint) {
                continue;
            }

            //Normalize workflow
            $normalizeWorkflow = $this->workflowHelper->normalize_workflow_jsonld($workflow);
            //Update arguments on normalized workflow so they are not persisted
            unset($arguments[$triggerParameter]);
            $requestBody = $this->workflowHelper->prepareWorkflowRequestBody($request, $apiEndpoint, is_null($data) ? $body : $data, null);
            $workflow_response = $this->workflowHelper->handleTrigger(Triggers::API_CALL, $requestBody, $normalizeWorkflow, $headerObj);
            $taskIds[] = json_decode($workflow_response, true)['task_id'];
        }
        // Added the below if just to maintain backward compatability of response.
        if (empty($taskIds)) {
            return new JsonResponse(array('message' => 'No active workflow matching the given route.'), Response::HTTP_NOT_FOUND);
        }
        else if (count($taskIds) == 1) {
            return JsonResponse::create(["task_id" => $taskIds[0]]);
        }
        return JsonResponse::create(["task_id" => $taskIds]);
    }

    protected function extractDataFromRequest($data)
    {
        if (array_key_exists("__resource", $data)) {
            $resourceShortName = $data['__resource'];
        } else {
            throw $this->createNotFoundException($message = 'Resource Not Found In Request');
        }
        $resourceClass = $this->getResourceFromClassName($resourceShortName);

        if (!($this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.' . $resourceShortName))) {
            throw new AccessDeniedException();
        }
        $resourceContext = ['resource_class' => $resourceClass];

        foreach ($data['records'] as $index => $record) {
            $object = $this->iriConverter->getItemFromIri($record, $resourceContext + ['fetch_data' => true]);
            $normalized_data = $this->serializer->normalize(
                $object,
                self::NORMALIZER_METHOD,
                $resourceContext
            );
            $data['records'][$index] = $normalized_data;
        }
        return $data;
    }

    private function fetch_workflow_data($workflowId)
    {
        $workflowRepository = $this->entityManager->getRepository(Workflow::class);
        $workflowObject = $workflowRepository->find($workflowId);
        if (!$workflowObject) {
            throw $this->createNotFoundException(
                'No workflow found for id ' . $workflowId
            );
        }
        return $this->workflowHelper->normalize_workflow_jsonld($workflowObject);
    }

    protected function prepareChangeData($updatedObject, $previousObject, $method, $playbook_iri, $playbook_name, $changeData = [])
    {
        $delta_data = [];
        $delta_data['oldData'] = $previousObject;
        $delta_data['newData'] = $updatedObject;
        $delta_data['changeData'] = $changeData;
        $userName = $this->extractUserName();
        return [
            self::DELTA_DATA => $delta_data,
            self::HTTP_METHOD => $method,
            self::REMOTE_ADDR => $_SERVER['REMOTE_ADDR'],
            self::COMPONENT => 'crudhub',
            self::USER_UUID => $this->getUser()->getUuid(),
            self::TRANSACTION_DATE => (int)($_SERVER['REQUEST_TIME_FLOAT'] * 1000),
            self::USER => $userName,
            self::PLAYBOOK_IRI => $playbook_iri,
            self::PLAYBOOK_NAME => $playbook_name
        ];
    }

    private function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }

    private function getHeaders($headers)
    {
        $header_list = array();
        foreach ($headers as $header => $value) {
            $header_list[$header] = $value[0];
        }
        return $header_list;
    }

    private function getUserId($loginId)
    {
        $client = $this->findHandlerForRoute('auth');
        $body = ["json" => ["loginid" => $loginId]];
        $userRequestUri = sprintf("%s/%s", $client->getTargetUri(), 'users');
        $response = null;
        try {
            $response = $client->request('POST', $userRequestUri, $body);
            $respBody = json_decode($response->getBody()->getContents(), true);
            return $respBody['uuid'];
        } catch (BadResponseException $exception) {
            return null;
        } catch (\Exception $e) {
            error_log('Failure: ' . $e->getMessage());
            return null;
        }
    }

    private function getuuid($userId)
    {
        $actorRepo = $this->entityManager->getRepository(Actor::class);
        try {
            $actor = $actorRepo->findOneBy(['userId' => $userId]);
            return $actor->getUuid();
        } catch (InvalidArgumentException | \Exception $e) {
            error_log($e->getMessage());
            return '';
        }
    }

    private function findHandlerForRoute($routeIdentifier)
    {
        $handler = $this->findHandlerMapForRoute($routeIdentifier);

        return $this->containers->get($handler[$routeIdentifier]['handlerName']);
    }

    private function findHandlerMapForRoute($routeIdentifier)
    {
        $handlerMap = $this->getParameter('app.proxy.handler.map');
        //$this->getParameter('app_proxy')['handlers'] this is workign

        if (!array_key_exists($routeIdentifier, $handlerMap)) {
            throw new NotFoundHttpException();
        }
        return $handlerMap;
    }
}
