<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\DataFixtures;

use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Persistence\ObjectManager;
use Nelmio\Alice\Loader\NativeLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SystemFixtures extends Fixture
{

    protected $containers;

    public function __construct(ContainerInterface $containers)
    {
        $this->containers = $containers;
    }

    // const JSON_FIXTURES = ['/ModelMetadata/picklist_names.json', '/ModelMetadata/module.json'];

    const JSON_FIXTURES = [
        'picklists.json',
        'systemSettings.json',
        '/SystemViewTemplate/cs_queue_management.json', '/SystemViewTemplate/app.json',
        '/SystemViewTemplate/grid_columns.json'
    ];

    const MODELMETADATA_FIXTURE_DIR = ['/ModelMetadata'];

    const YML_FILES = ['modulesRolesFixtures.yml', '/Workflow/WorkflowStepType.yml', '/Workflow/WorkflowCollections.yml'];

    const JS_FILES = [
        '/ViewTemplate/reporting.json', '/ViewTemplate/dashboard.json',
        '/SystemViewTemplate/modules/agents/detail.json', '/SystemViewTemplate/modules/agents/form.json', '/SystemViewTemplate/modules/agents/list.json',
        '/SystemViewTemplate/modules/comments/detail.json', '/SystemViewTemplate/modules/comments/list.json', '/SystemViewTemplate/modules/comments/form.json',
        '/SystemViewTemplate/modules/tasks/detail.json', '/SystemViewTemplate/modules/tasks/list.json', '/SystemViewTemplate/modules/tasks/form.json',
        '/SystemViewTemplate/modules/scans/detail.json', '/SystemViewTemplate/modules/scans/list.json', '/SystemViewTemplate/modules/scans/form.json',
        '/SystemViewTemplate/modules/indicators/detail.json', '/SystemViewTemplate/modules/indicators/list.json', '/SystemViewTemplate/modules/indicators/form.json',
        '/SystemViewTemplate/modules/emails/detail.json', '/SystemViewTemplate/modules/emails/list.json', '/SystemViewTemplate/modules/emails/form.json',
        '/SystemViewTemplate/modules/tenants/detail.json', '/SystemViewTemplate/modules/tenants/list.json', '/SystemViewTemplate/modules/tenants/form.json',
        '/SystemViewTemplate/modules/sla_template/detail.json', '/SystemViewTemplate/modules/sla_template/list.json', '/SystemViewTemplate/modules/sla_template/form.json',
        '/SystemViewTemplate/modules/alerts/detail.json', '/SystemViewTemplate/modules/alerts/list.json', '/SystemViewTemplate/modules/alerts/form.json',
        '/SystemViewTemplate/modules/notes/detail.json', '/SystemViewTemplate/modules/notes/list.json', '/SystemViewTemplate/modules/notes/form.json',
        '/SystemViewTemplate/modules/events/detail.json', '/SystemViewTemplate/modules/events/list.json', '/SystemViewTemplate/modules/events/form.json',
        '/SystemViewTemplate/modules/approvals/detail.json', '/SystemViewTemplate/modules/approvals/list.json', '/SystemViewTemplate/modules/approvals/form.json',
        '/SystemViewTemplate/modules/attachments/detail.json', '/SystemViewTemplate/modules/attachments/list.json', '/SystemViewTemplate/modules/attachments/form.json',
        '/SystemViewTemplate/modules/assets/detail.json', '/SystemViewTemplate/modules/assets/list.json', '/SystemViewTemplate/modules/assets/form.json',
        '/SystemViewTemplate/modules/warrooms/detail.json', '/SystemViewTemplate/modules/warrooms/list.json', '/SystemViewTemplate/modules/warrooms/form.json',
        '/SystemViewTemplate/modules/vulnerabilities/detail.json', '/SystemViewTemplate/modules/vulnerabilities/list.json', '/SystemViewTemplate/modules/vulnerabilities/form.json',
        '/SystemViewTemplate/modules/companies/detail.json', '/SystemViewTemplate/modules/companies/list.json', '/SystemViewTemplate/modules/companies/form.json',
        '/SystemViewTemplate/modules/widgets/detail.json', '/SystemViewTemplate/modules/widgets/list.json', '/SystemViewTemplate/modules/widgets/form.json',
        '/SystemViewTemplate/modules/incidents/detail.json', '/SystemViewTemplate/modules/incidents/list.json', '/SystemViewTemplate/modules/incidents/form.json',
        '/Workflow/ReportManagementWorkflow/workflows.json', '/Workflow/ApprovalWorkflows/workflows.json', '/Workflow/SLAWorkflow/workflows.json',
        '/Workflow/UtilitiesWorkflow/workflows.json', '/Workflow/SchedulerSystemWorkflow/workflows.json', '/Workflow/SystemWorkflow/workflows.json', '/Workflow/WarRoomWorkflow/workflows.json',
        '/EmailTemplates/EmailTemplate.json', '/SystemViewTemplate/modules/announcements/detail.json', '/SystemViewTemplate/modules/announcements/list.json', '/SystemViewTemplate/modules/announcements/form.json'
    ];

    public function load(ObjectManager $manager)
    {

        $loader = new \Nelmio\Alice\Loader\NativeLoader();
        foreach (self::YML_FILES as $file) {
            $ymlSet = $loader->loadFile(__DIR__ . '/' . $file);
            foreach ($ymlSet->getObjects() as $key => $value) {
                $manager->persist($ymlSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        $listOfMmds = array_diff(scandir(__DIR__ . '/ModelMetadata'), array('.', '..'));
        foreach (self::JSON_FIXTURES as $file) {
            $jsonSet = $loader->loadFile(__DIR__ . '/' . $file);
            foreach ($jsonSet->getObjects() as $key => $value) {
                $manager->persist($jsonSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        foreach ($listOfMmds as $file) {
            $jsonSet = $loader->loadFile(__DIR__ . '/ModelMetadata/' . $file);
            foreach ($jsonSet->getObjects() as $key => $value) {
                $manager->persist($jsonSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        $this->loadJsonFixtures($manager);
    }

    public function loadJsonFixtures($manager)
    {
        $serialize = $this->containers->get('serializer');
        foreach (self::JS_FILES as $file) {
            $file_content = json_decode(file_get_contents(__DIR__ . $file), true);
            foreach ($file_content as $key => $values) {
                foreach ($values as $value) {
                    $object = $serialize->deserialize(
                        json_encode($value),
                        $key,
                        'jsonld',
                        ['fetch_data' => false, "ignore_authorization" => true, "ignore_field_authorization" => true, "ignore_relationship_authorization" => true]
                    );
                    $manager->persist($object);
                }
                $manager->flush();
            }
        }
    }
}
