<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
// api/src/Doctrine/CurrentUserExtension.php

namespace App\Doctrine;

use ApiPlatform\Core\Bridge\Doctrine\Orm\Extension\QueryCollectionExtensionInterface;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Extension\QueryItemExtensionInterface;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Util\QueryNameGeneratorInterface;
use App\Doctrine\ORM\FilterQueryBuilder;
use App\Doctrine\ORM\SearchQueryBuilder;
use App\Providers\QueryProvider;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\Security;

final class QueryExtension implements QueryCollectionExtensionInterface, QueryItemExtensionInterface
{
    private $security;

    private $requeststack;

    private $request;

    private $filterQueryBuilder;

    protected $queryProvider;

    private $searchQueryBuilder;

    public function __construct(Security $security, RequestStack $requeststack, FilterQueryBuilder $filterQueryBuilder, QueryProvider $queryProvider, SearchQueryBuilder $searchQueryBuilder)
    {
        $this->security = $security;
        $this->requeststack = $requeststack;
        $this->filterQueryBuilder = $filterQueryBuilder;
        $this->queryProvider = $queryProvider;
        $this->searchQueryBuilder = $searchQueryBuilder;
    }

    private function setCurrentRequest()
    {
        $this->request = $this->requeststack->getCurrentRequest();
    }

    public function applyToCollection(QueryBuilder $queryBuilder, QueryNameGeneratorInterface $queryNameGenerator, string $resourceClass, string $operationName = null): void
    {
        $this->setCurrentRequest();
        $this->addWhere($queryBuilder, $resourceClass);
    }

    public function applyToItem(QueryBuilder $queryBuilder, QueryNameGeneratorInterface $queryNameGenerator, string $resourceClass, array $identifiers, string $operationName = null, array $context = []): void
    {
        $this->setCurrentRequest();
        $this->addWhere($queryBuilder, $resourceClass);
    }

    private function addWhere(QueryBuilder $queryBuilder, string $resourceClass): void
    {
        $content = $this->request->getContent();
        $query = json_decode($content, true);
        $query = $this->queryProvider->getQueryObject();
        if (!(isset($query['logic']) && isset($query['filters']))) {
            $query['logic'] = "AND";
            $query['filters'] = [];
        }
        $this->filterQueryBuilder->fromQueryObject($query, $queryBuilder, $resourceClass);
        $this->searchQueryBuilder->fromQueryObject($query, $queryBuilder, $resourceClass);
    }
}
