<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder;

use App\Providers\JwtLogoutKeyProvider;
use Lexik\Bundle\JWTAuthenticationBundle\Encoder\LcobucciJWTEncoder;
use Lexik\Bundle\JWTAuthenticationBundle\Services\KeyLoader\KeyLoaderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class CorrectJwtEncoder extends LcobucciJWTEncoder
{
    protected $container;
    protected $jwsProvider;
    protected $jwtLogoutKeyProvider;

    const ALGORITHM = 'RS256';

    public function __construct(ContainerInterface $container, KeyLoaderInterface $keyLoader, JwtLogoutKeyProvider $jwtLogoutKeyProvider)
    {
        $this->container = $container;
        $this->keyLoader = $keyLoader;
        $this->jwtLogoutKeyProvider =  $jwtLogoutKeyProvider;
    }

    public function decode($token)
    {
        try {
            $jws = CorrectSimpleJWS::load($token);
        } catch (\InvalidArgumentException $e) {
            return false;
        } catch (\Exception $err) {
            return false;
        }

        if ($jws instanceof CorrectSimpleJWS && !$jws->isValid($this->getPublicKey(), self::ALGORITHM)) {
            return false;
        }

        if ($this->isLogout($token)) {
            return false;
        }

        return $jws->getPayload();
    }

    /**
     *
     */
    protected function getPublicKey()
    {
        while (!empty($errorStr = openssl_error_string())) {
        }
        $pkey = $this->keyLoader->loadKey("public");
        while (!empty($errorStr = openssl_error_string())) {
        }
        return $pkey;
    }

    /**
     * @param string $jwtKey
     * @return bool
     */
    private function isLogout($jwtKey)
    {
        if ($this->container == null) {
            return false;
        }
        return $this->jwtLogoutKeyProvider->isJwtLogoutKey("Bearer " . $jwtKey);
    }
}
