<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder;

use Namshi\JOSE\JWS;

class CorrectSimpleJWS extends JWS
{

    public function isValid($key, $algo = null, $type = null)
    {
        $payload = $this->getPayload();
        return $this->verify($key, $algo) && !$this->isExpired($payload) && $this->isValidType($payload, $type);
    }

    public function isValidType($payload, $type)
    {
        if (!isset($payload['type'])){
            return false;
        }
        $tokenType = $payload['type'];
        if (is_null($type)){
            # block at least restricted and noauth
            if ($tokenType === 'restricted' || $tokenType === 'noauth' || $tokenType === '2fa'){
                return false;
            } else {
                return true;
            }
        }
        else if ($tokenType === $type) {
            return true;
        }
        return false;
    }
 
    /**
     * @param $payload
     * @return bool
     */
    public function isExpired($payload)
    {
        if (isset($payload['exp']) && is_numeric($payload['exp'])) {
            $now = new \DateTime('now');
            return ($now->format('U') - $payload['exp']) > 0;
        }
        return false;
    }

}
