<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder\Hmac;

use App\Encoder\Hmac\DasHmacSecretKeyProvider;
use App\Providers\FingerprintProvider;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\Authentication\Provider\AuthenticationProviderInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Symfony\Component\Security\Core\User\UserProviderInterface;


class HmacProvider implements AuthenticationProviderInterface
{
    protected $requestStack;

    /** @var DasHmacSecretKeyProvider */
    protected $dasHmacSecretKeyProvider;

    /** @var UserProviderInterface */
    protected $userProvider;

    /**
     * @param UserProviderInterface $userProvider
     */
    public function __construct(UserProviderInterface $userProvider, RequestStack $requestStack, DasHmacSecretKeyProvider $dasHmacSecretKeyProvider)
    {
        $this->userProvider = $userProvider;
        $this->requestStack = $requestStack;
        $this->dasHmacSecretKeyProvider = $dasHmacSecretKeyProvider;
    }
  

    /**
     * {@inheritdoc}
     */
    public function authenticate(TokenInterface $token)
    {
        if (!$this->validateHmacFingerprint($token->getCredentials())) {
            throw new AuthenticationException('Invalid HMAC Fingerprint');
        }

        $userId = $this->dasHmacSecretKeyProvider->getUserId();
        $user = $this->userProvider->loadUserByUsername($userId);

        $authToken = new HmacUserToken($user->getRoles(true, true));
        $authToken->setUser($user);
        $authToken->setPayload($token->getCredentials());

        return $authToken;
    }

    /**
     * {@inheritdoc}
     */
    public function supports(TokenInterface $token)
    {
        return $token instanceof HmacUserToken;
    }

    /**
     * @param string $hmacFingerprint
     * @return bool
     */
    protected function validateHmacFingerprint($hmacFingerprint)
    {
        $request = $this->requestStack->getCurrentRequest();
        $success = FingerprintProvider::validateFingerprint($hmacFingerprint, $this->dasHmacSecretKeyProvider, $request);
        return $success;
    }
}
