<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder\Restricted;

use App\Service\UtilityService;
use App\Providers\JwtLogoutKeyProvider;
use App\Providers\RestrictedFingerprintProvider;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Lexik\Bundle\JWTAuthenticationBundle\Services\KeyLoader\KeyLoaderInterface;
use Symfony\Component\Security\Core\Authentication\Provider\AuthenticationProviderInterface;


class RestrictedProvider implements AuthenticationProviderInterface
{
    protected $requestStack;

    /** @var UserProviderInterface */
    protected $userProvider;

    private $keyLoader;
    private $jwtLogoutKeyProvider;
    private $utilityService;

    /**
     * @param UserProviderInterface $userProvider
     */
    public function __construct(
        UserProviderInterface $userProvider,
        RequestStack $requestStack,
        KeyLoaderInterface $keyLoader,
        JwtLogoutKeyProvider $jwtLogoutKeyProvider,
        UtilityService $utilityService
    ) {
        $this->userProvider = $userProvider;
        $this->requestStack = $requestStack;
        $this->keyLoader = $keyLoader;
        $this->jwtLogoutKeyProvider =  $jwtLogoutKeyProvider;
        $this->utilityService = $utilityService;
    }


    /**
     * {@inheritdoc}
     */
    public function authenticate(TokenInterface $token)
    {
        $decodedFingerPrint = $this->validateRestrictedFingerprint($token->getCredentials());
        $authToken = new RestrictedUserToken();
        $authToken->setPayload($token->getCredentials());
        $authToken->setEmail($decodedFingerPrint['email']);
        return $authToken;
    }

    /**
     * {@inheritdoc}
     */
    public function supports(TokenInterface $token)
    {
        return $token instanceof RestrictedUserToken;
    }

    /**
     * @param string $hmacFingerprint
     * @return bool
     */
    protected function validateRestrictedFingerprint($restrictedFingerprint)
    {
        $request = $this->requestStack->getCurrentRequest();
        $restrictedProvider = new RestrictedFingerprintProvider($this->keyLoader, $this->jwtLogoutKeyProvider, $this->utilityService);
        $success = $restrictedProvider->validateFingerprint($restrictedFingerprint, $request);
        return $success;
    }
}
