<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Actors;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Actors\Actor;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * Appliance
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="appliances", "fsr_module"="appliances", "fsr_singular"="Appliance"},
 *     denormalizationContext={"groups"={"fsr_all"}, "fetch_data"=false},
 *     normalizationContext={"max_depth"=2},
 *     collectionOperations={
 *          "get" = { "path"="appliances", "security_post_denormalize" = "is_granted('read.appliances', object)" },
 *          "post" = { "path"="appliances", "security_post_denormalize" = "is_granted('create.appliances', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "path"="appliances/{id}", "security" = "is_granted('read.appliances', object)" },
 *          "put" = { "path"="appliances/{id}", "security" = "is_granted('update.appliances', object)" },
 *          "delete" = { "path"="appliances/{id}", "security" = "is_granted('delete.appliances', object)" }
 *     }
 * )
 * @Doctrine\ORM\Mapping\Table(name="actors")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 */
class Appliance extends Actor
{
    
    use \App\Traits\Trackable;
    use \App\Traits\Indexable;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $name;

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }
}
