<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Actors;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Actors\Actor;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * Person
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="people", "fsr_module"="people", "fsr_singular"="Person"},
 *     denormalizationContext={"groups"={"fsr_all"}, "fetch_data"=false},
 *     normalizationContext={"max_depth"=2},
 *     collectionOperations={
 *          "get" = { "path"="people", "security_post_denormalize" = "is_granted('read.people', object)" },
 *          "post" = { "path"="people", "security_post_denormalize" = "is_granted('create.people', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "path"="people/{id}", "security" = "is_granted('read.people', object)" },
 *          "put" = { "path"="people/{id}", "security" = "is_granted('update.people', object)" },
 *          "delete" = { "path"="people/{id}", "security" = "is_granted('delete.people', object)" }
 *     }
 * )
 * @Doctrine\ORM\Mapping\Table(name="actors")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 */
class Person extends Actor
{
    
    use \App\Traits\Trackable;
    use \App\Traits\Indexable;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $firstname;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $lastname;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=50, nullable=true)
     */
    private $title;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $department;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $email;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="text", nullable=true)
     */
    private $description;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=50, nullable=true)
     */
    private $phoneFax;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=15, nullable=true)
     */
    private $phoneHome;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=15, nullable=true)
     */
    private $phoneMobile;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=15, nullable=true)
     */
    private $phoneWork;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $csActive;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=36, nullable=true)
     */
    private $accessType;

    public function getFirstname(): ?string
    {
        return $this->firstname;
    }

    public function setFirstname(?string $firstname): self
    {
        $this->firstname = $firstname;

        return $this;
    }

    public function getLastname(): ?string
    {
        return $this->lastname;
    }

    public function setLastname(?string $lastname): self
    {
        $this->lastname = $lastname;

        return $this;
    }

    public function getTitle(): ?string
    {
        return $this->title;
    }

    public function setTitle(?string $title): self
    {
        $this->title = $title;

        return $this;
    }

    public function getDepartment(): ?string
    {
        return $this->department;
    }

    public function setDepartment(?string $department): self
    {
        $this->department = $department;

        return $this;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(?string $email): self
    {
        $this->email = $email;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getPhoneFax(): ?string
    {
        return $this->phoneFax;
    }

    public function setPhoneFax(?string $phoneFax): self
    {
        $this->phoneFax = $phoneFax;

        return $this;
    }

    public function getPhoneHome(): ?string
    {
        return $this->phoneHome;
    }

    public function setPhoneHome(?string $phoneHome): self
    {
        $this->phoneHome = $phoneHome;

        return $this;
    }

    public function getPhoneMobile(): ?string
    {
        return $this->phoneMobile;
    }

    public function setPhoneMobile(?string $phoneMobile): self
    {
        $this->phoneMobile = $phoneMobile;

        return $this;
    }

    public function getPhoneWork(): ?string
    {
        return $this->phoneWork;
    }

    public function setPhoneWork(?string $phoneWork): self
    {
        $this->phoneWork = $phoneWork;

        return $this;
    }

    public function getCsActive(): ?bool
    {
        return $this->csActive;
    }

    public function setCsActive(?bool $csActive): self
    {
        $this->csActive = $csActive;

        return $this;
    }

    /**
     * Get the value of accessType
     */ 
    public function getAccessType()
    {
        return $this->accessType;
    }

    /**
     * Set the value of accessType
     *
     * @return  self
     */ 
    public function setAccessType($accessType)
    {
        $this->accessType = $accessType;

        return $this;
    }
}
