<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Entity\Core\ExportTemplate;
use App\Repository\ExportJobRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="export_jobs", "fsr_module"="security", "fsr_singular"="Export Job"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" },
 *          "put" = { "security" = "is_granted('update.security', object)" },
 *          "delete" = { "security" = "is_granted('delete.security', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=ExportJobRepository::class)
 * @ORM\Table(name="export_jobs", indexes={@Doctrine\ORM\Mapping\Index(name="export_jobs_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="export_jobs_id_search_idx", columns={"id"})})
 */
class ExportJob extends BaseEntity
{

    use \App\Traits\Trackable;
    use \App\Traits\Indexable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "file"})
     * @ORM\Column(type="text", nullable=true)
     */
    private $file;

    /**
     * @Groups({"fsr_primary", "fsr_all", "status"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $status;

    /**
     * @Groups({"fsr_primary", "fsr_all", "errorMessage"})
     * @ORM\Column(type="text", nullable=true)
     */
    private $errorMessage = '';

    /**
     * @Groups({"fsr_primary", "fsr_all", "fileName"})
     * @ORM\Column(name="`fileName`", type="string", length=1024, nullable=true, unique=false)
     */
    private $fileName = '';

    /**
     * @Groups({"fsr_primary", "fsr_all", "progressPercent"})
     * @ORM\Column(type="integer", nullable=false)
     */
    private $progressPercent = 0;

    /**
     * @Groups({"fsr_all", "template"})
     * @ORM\ManyToOne(targetEntity="App\Entity\Core\ExportTemplate",cascade={"persist"}, inversedBy="exportJobs")
     * @ORM\JoinColumn(name="template", referencedColumnName="uuid")
     */
    private $template;


    public function getFile(): ?string
    {
        return $this->file;
    }

    public function setFile(?string $file): self
    {
        $this->file = $file;

        return $this;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(?string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getErrorMessage(): ?string
    {
        return $this->errorMessage;
    }

    public function setErrorMessage(?string $errorMessage): self
    {
        $this->errorMessage = $errorMessage;

        return $this;
    }

    public function getFileName(): ?string
    {
        return $this->fileName;
    }

    public function setFileName(string $fileName): self
    {
        $this->fileName = $fileName;

        return $this;
    }

    public function getProgressPercent(): ?int
    {
        return $this->progressPercent;
    }

    public function setProgressPercent(?int $progressPercent): self
    {
        $this->progressPercent = $progressPercent;

        return $this;
    }

    /**
     * @return ExportTemplate
     */
    public function getTemplate()
    {
        return $this->template;
    }

    public function setTemplate(ExportTemplate $template = null)
    {
        $this->template = $template;

        return $this;
    }
}
