<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ModelMetadata;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\ModelMetadata\AttribModelMetadata;
use App\Validator\KeywordConstraint;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="model_metadatas", "fsr_module"="application", "fsr_singular"="model_metadata"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass="\Doctrine\ORM\EntityRepository")
 * @ORM\Table(name="model_metadatas", uniqueConstraints={@UniqueConstraint(name="field_model_uniques", columns={"type"})})
 */
class ModelMetadata extends Metadatas
{

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @KeywordConstraint()
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $type;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @KeywordConstraint()
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $parentType;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @KeywordConstraint()
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $tableName;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $ownable;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $userOwnable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $trackable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = true})
     */
    private $taggable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $peerReplicable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $defaultSort = [];

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $uniqueConstraint = [];

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $indexable = true;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", options={"default" = false})
     */
    private $writable = true;

    /**
     * @Groups({"fsr_all"})
     * @ORM\OneToMany(targetEntity="AttribModelMetadata", mappedBy="sattrib", cascade={"persist", "remove"})
     */
    private $attributes;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $system = false;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $module;

    public function __construct()
    {
        $this->attributes = new ArrayCollection();
        $this->queries = new ArrayCollection();
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getParentType(): ?string
    {
        return $this->parentType;
    }

    public function setParentType(?string $parentType): self
    {
        $this->parentType = $parentType;

        return $this;
    }

    public function getTableName(): ?string
    {
        return $this->tableName;
    }

    public function setTableName(?string $tableName): self
    {
        $this->tableName = $tableName;

        return $this;
    }

    public function getOwnable(): ?bool
    {
        return $this->ownable;
    }

    public function setOwnable(bool $ownable): self
    {
        $this->ownable = $ownable;

        return $this;
    }

    public function getTrackable(): ?bool
    {
        return $this->trackable;
    }

    public function setTrackable(bool $trackable): self
    {
        $this->trackable = $trackable;

        return $this;
    }

    public function getTaggable(): ?bool
    {
        return $this->taggable;
    }

    public function setTaggable(bool $taggable): self
    {
        $this->taggable = $taggable;

        return $this;
    }

    public function getPeerReplicable(): ?bool
    {
        return $this->peerReplicable;
    }

    public function setPeerReplicable(bool $peerReplicable): self
    {
        $this->peerReplicable = $peerReplicable;

        return $this;
    }

    public function getDefaultSort(): ?array
    {
        return $this->defaultSort;
    }

    public function setDefaultSort(?array $defaultSort): self
    {
        $this->defaultSort = $defaultSort;

        return $this;
    }

    public function getUniqueConstraint(): ?array
    {
        return $this->uniqueConstraint;
    }

    public function setUniqueConstraint(?array $uniqueConstraint): self
    {
        $this->uniqueConstraint = $uniqueConstraint;

        return $this;
    }

    public function getIndexable(): ?bool
    {
        return $this->indexable;
    }

    public function setIndexable(bool $indexable): self
    {
        $this->indexable = $indexable;

        return $this;
    }

    public function getWritable(): ?bool
    {
        return $this->writable;
    }

    public function setWritable(bool $writable): self
    {
        $this->writable = $writable;

        return $this;
    }

    /**
     * @return Collection|AttribModelMetadata[]
     */
    public function getAttributes(): Collection
    {
        return $this->attributes;
    }

    public function addAttribute(AttribModelMetadata $attribute): self
    {
        if (!$this->attributes->contains($attribute)) {
            $this->attributes[] = $attribute;
            $attribute->setSattrib($this);
        }

        return $this;
    }

    public function removeAttribute(AttribModelMetadata $attribute): self
    {
        if ($this->attributes->contains($attribute)) {
            $this->attributes->removeElement($attribute);
            // set the owning side to null (unless already changed)
            if ($attribute->getSattrib() === $this) {
                $attribute->setSattrib(null);
            }
        }

        return $this;
    }

    /**
     * Get the value of system
     */
    public function getSystem()
    {
        return $this->system;
    }

    /**
     * Set the value of system
     *
     * @return  self
     */
    public function setSystem($system)
    {
        $this->system = $system;

        return $this;
    }

    /**
     * Get the value of module
     */
    public function getModule()
    {
        return $this->module;
    }

    /**
     * Set the value of module
     *
     * @return  self
     */
    public function setModule($module)
    {
        if (!is_null($module)) {
            $this->module = $module;
            return $this;
        }
        $this->module = $this->getType();
        return $this;
    }

    /**
     * Get the value of userOwnable
     */
    public function getUserOwnable()
    {
        return $this->userOwnable;
    }

    /**
     * Set the value of userOwnable
     *
     * @return  self
     */
    public function setUserOwnable($userOwnable)
    {
        $this->userOwnable = $userOwnable;

        return $this;
    }
}
