<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Settings;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\SystemSettingsRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="system_settings", "fsr_module"="application", "fsr_singular"="system_setting"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\SystemSettingsRepository")
 * @ORM\Table(name="cyops.system_settings", schema="cyops", uniqueConstraints={@UniqueConstraint(name="restriction_unique", columns={"restriction"})})
 */
class SystemSettings extends Settings
{

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $privateValues;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $globalValues;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $restriction;

    public function getPrivateValues(): ?array
    {
        return $this->privateValues;
    }

    public function setPrivateValues(?array $privateValues): self
    {
        $this->privateValues = $privateValues;

        return $this;
    }

    public function getRestriction(): ?string
    {
        return $this->restriction;
    }

    public function setRestriction(?string $restriction): self
    {
        $this->restriction = $restriction;

        return $this;
    }

    /**
     * Get the value of globalValues
     *
     * @return  array
     */ 
    public function getGlobalValues()
    {
        return $this->globalValues;
    }

    /**
     * Set the value of globalValues
     *
     * @param  array  $globalValues
     *
     * @return  self
     */ 
    public function setGlobalValues($globalValues)
    {
        $this->globalValues = $globalValues;

        return $this;
    }
}
