<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Workflow;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\RuleRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="rules", "fsr_module"="rules", "fsr_singular"="rule"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.rules', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.rules', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.rules', object)" },
 *          "put" = { "security" = "is_granted('update.rules', object)" },
 *          "delete" = { "security" = "is_granted('delete.rules', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=RuleRepository::class)
 * @ORM\Table(name="rules", uniqueConstraints={@UniqueConstraint(name="name_collection_unique", columns={"name", "collection"})})
 */
class Rule extends BaseEntity
{
    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "tag"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $tag;

    /**
     * @Groups({"fsr_primary", "fsr_all", "isActive"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $isActive;

    /**
     * @Groups({"fsr_primary", "fsr_all", "trigger"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $trigger = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "condition"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $condition = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "actions"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $actions = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "collection"})
     * @ORM\ManyToOne(targetEntity=RuleCollection::class, inversedBy="rules")
     * @ORM\JoinColumn(name="collection", nullable=true, referencedColumnName="uuid")
     */
    private $collection;


    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getTag(): ?string
    {
        return $this->tag;
    }

    public function setTag(?string $tag): self
    {
        $this->tag = $tag;

        return $this;
    }

    public function getIsActive(): ?bool
    {
        return $this->isActive;
    }

    public function setIsActive(?bool $isActive): self
    {
        $this->isActive = $isActive;

        return $this;
    }

    public function getTrigger(): ?array
    {
        return $this->trigger;
    }

    public function setTrigger(?array $trigger): self
    {
        $this->trigger = $trigger;

        return $this;
    }

    public function getCondition(): ?array
    {
        return $this->condition;
    }

    public function setCondition(?array $condition): self
    {
        $this->condition = $condition;

        return $this;
    }

    public function getActions(): ?array
    {
        return $this->actions;
    }

    public function setActions(?array $actions): self
    {
        $this->actions = $actions;

        return $this;
    }

    public function getCollection(): ?RuleCollection
    {
        return $this->collection;
    }

    public function setCollection(?RuleCollection $collection): self
    {
        $this->collection = $collection;

        return $this;
    }
}
