<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Workflow;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\RuleCollectionRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="rule_collections", "fsr_module"="rules", "fsr_singular"="rule_collection"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.rules', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.rules', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.rules', object)" },
 *          "put" = { "security" = "is_granted('update.rules', object)" },
 *          "delete" = { "security" = "is_granted('delete.rules', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=RuleCollectionRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="rule_collections")
 */
class RuleCollection extends BaseEntity
{


    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=false, unique=true)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "description"})
     * @ORM\Column(type="text", nullable=true)
     */
    private $description;

    /**
     * @Groups({"fsr_primary", "fsr_all", "rules"})
     * @ORM\OneToMany(targetEntity=Rule::class, mappedBy="collection", orphanRemoval=true, cascade={"persist", "remove"})
     */
    private $rules;

    public function __construct()
    {
        $this->rules = new ArrayCollection();
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    /**
     * @return Collection|Rule[]
     */
    public function getRules(): Collection
    {
        return $this->rules;
    }

    public function addRule(Rule $rule): self
    {
        if (!$this->rules->contains($rule)) {
            $this->rules[] = $rule;
            $rule->setCollection($this);
        }

        return $this;
    }

    public function removeRule(Rule $rule): self
    {
        if ($this->rules->contains($rule)) {
            $this->rules->removeElement($rule);
            // set the owning side to null (unless already changed)
            if ($rule->getCollection() === $this) {
                $rule->setCollection(null);
            }
        }

        return $this;
    }

    /**
     * @Groups({"workflowCount"})
    */
    public function getWorkflowCount()
    {
    return $this->rules->count();
    }

}
