<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Workflow;

use ApiPlatform\Core\Annotation\ApiProperty;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Entity\Workflow\WorkflowStep;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="workflow_routes", "fsr_module"="workflows", "fsr_singular"="workflow_route"},
 *     normalizationContext={"ignore_depth"=true},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.workflows', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.workflows', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.workflows', object)" },
 *          "put" = { "security" = "is_granted('update.workflows', object)" },
 *          "delete" = { "security" = "is_granted('delete.workflows', object)" }
 *     }
 * )
 * @ORM\Table(name="workflow_routes")
 * @ORM\Entity()
 */
class WorkflowRoute extends BaseEntity
{

    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "targetStep"})
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="WorkflowStep")
     * @ORM\JoinColumn(name="targetstep_uuid", referencedColumnName="uuid", nullable=false, onDelete="CASCADE")
     * @ApiProperty(readableLink=false, writableLink=false)
     */
    protected $targetStep;

    /**
     * @Groups({"fsr_primary", "fsr_all", "sourceStep"})
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="WorkflowStep")
     * @ORM\JoinColumn(name="sourcestep_uuid", referencedColumnName="uuid", nullable=false, onDelete="CASCADE")
     * @ApiProperty(readableLink=false, writableLink=false)
     */
    protected $sourceStep;

    /**
     * @Groups({"fsr_primary", "fsr_all", "label"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $label;

    /**
     * @Groups({"fsr_primary", "fsr_all", "isExecuted"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $isExecuted = false;

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getLabel(): ?string
    {
        return $this->label;
    }

    public function setLabel(?string $label): self
    {
        $this->label = $label;

        return $this;
    }

    public function getIsExecuted(): ?bool
    {
        return $this->isExecuted;
    }

    public function setIsExecuted(?bool $isExecuted): self
    {
        $this->isExecuted = $isExecuted;

        return $this;
    }

    /**
     * Set targetStep
     *
     * @param \App\Entity\Workflow\WorkflowStep $targetStep
     * @return WorkflowRoute
     */
    public function setTargetStep(WorkflowStep $targetStep)
    {
        $this->targetStep = $targetStep;

        return $this;
    }

    /** @return \App\Entity\Workflow\WorkflowStep */
    public function getTargetStep()
    {
        return $this->targetStep;
    }

    /**
     * Set sourceStep
     *
     * @param \App\Entity\Workflow\WorkflowStep $sourceStep
     * @return WorkflowRoute
     */
    public function setSourceStep(WorkflowStep $sourceStep)
    {
        $this->sourceStep = $sourceStep;

        return $this;
    }

    /** @return \App\Entity\Workflow\WorkflowStep */
    public function getSourceStep()
    {
        return $this->sourceStep;
    }
}
