<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Custom\Comment;
use Doctrine\Persistence\Event\LifecycleEventArgs;
use Symfony\Component\HttpFoundation\RequestStack;

class CommentSubscriber
{

    private $requestStack;

    public function __construct(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    /**
     * @throws \Exception
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        if ($this->requestStack->getCurrentRequest() && $this->requestStack->getCurrentRequest()->attributes->get('fsr_subrequest_type') === 'import') {
            return false;
        }
        $entity = $args->getObject();
        if ($entity instanceof Comment){
            $currentDate = new \DateTime('NOW', new \DateTimeZone('UTC'));
            $replyTo = $entity->getReplyTo();
            if ($replyTo) {
                $replyTo->setLastReplyDate($currentDate);
            } else {
                $entity->setLastReplyDate($currentDate);
            }
        }
        
    }

    /**
     * @throws \Exception
     */
    public function preUpdate(LifecycleEventArgs $args)
    {
        if ($this->requestStack->getCurrentRequest() && $this->requestStack->getCurrentRequest()->attributes->get('fsr_subrequest_type') === 'import') {
            return false;
        }
        $entity = $args->getObject();
        if ($entity instanceof Comment){
            if ($entity->getIsDeleted() != true) {
                $replyTo = $entity->getReplyTo();
                if ($replyTo) {
                    $currentDate = new \DateTime('NOW', new \DateTimeZone('UTC'));
                    $replyTo->setModifyDate($currentDate);
                }
            }
        }
    }
}
