<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Core\PgFile;
use InvalidArgumentException;
use App\Entity\Base\BaseEntity;
use App\Service\MetadataHelper;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class FileDeleteSubscriber implements EventSubscriberInterface
{
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;

    /** @var ContainerInterface */
    protected $containers;
    protected $metadataHelper;

    private $inflector;


    public function __construct(ContainerInterface $containers, IriConverterInterface $iriConverter, EntityManagerInterface $entityManager, MetadataHelper $metadataHelper)
    {

        $this->containers = $containers;
        $this->iriConverter = $iriConverter;
        $this->inflector = InflectorFactory::create()->build();
        $this->entityManager = $entityManager;
        $this->metadataHelper = $metadataHelper;
    }


    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['FileDelete', 33]
        ];
    }

    public function FileDelete(ViewEvent $event): void
    {
        $object = $event->getControllerResult();
        $request = $event->getRequest();
        $method = $request->getMethod();
        if (($method && $method != Request::METHOD_DELETE) || !$object instanceof BaseEntity) {
            return;
        }
        $entityClass = get_class($object);
        $entityFieldMap = $this->metadataHelper->getEntityFields($entityClass) ?? null;
        if ($entityFieldMap && count($entityFieldMap['fileFields'])) { // Added extra check because of missing type map, can be removed once all entity are mapped 
            foreach ($entityFieldMap['fileFields'] as $key => $value) {
                $camelized = $this->inflector->camelize($value);
                $getMethod = 'get' . $camelized;
                if ($object->{$getMethod}()) {
                    try {
                        $fileObject = $this->iriConverter->getItemFromIri($object->{$getMethod}());
                        // $value = $this->normalize($fileObject, 'jsonld', $context);
                        $pgFileRepo = $this->entityManager->getRepository(PgFile::class);
                        if ($fileObject->getFile()) {
                            $file_explode = explode('/', $fileObject->getFile());
                            $pgFileId = end($file_explode);
                            $pgFile = $pgFileRepo->findOneBy(array("uuid" => $pgFileId));
                            $this->entityManager->remove($pgFile);
                            $this->entityManager->flush();
                            $this->entityManager->remove($fileObject);
                            $this->entityManager->flush();
                        }
                    } catch (InvalidArgumentException | \Exception $e) {
                        $value = null;
                    }
                }
            }
        }
    }
}
