<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Traits\Ownable;
use App\Traits\UserOwnable;
use InvalidArgumentException;
use App\Entity\Base\BaseEntity;
use App\Service\MetadataHelper;
use Doctrine\ORM\EntityManager;
use Doctrine\Inflector\Inflector;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;

final class FileOwnershipCascadingSubscriber implements EventSubscriberInterface
{
    protected $iriConverter;

    protected $requeststack;

    protected $propertyAccessor;

    /** @var EntityManager */
    protected $entityManager;
    protected $serializer;

    /** @var ContainerInterface */
    protected $container;
    protected $metadataHelper;

    private $inflector;
    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['FileOwnership', 31]
        ];
    }

    public function __construct(IriConverterInterface $iriConverter, SerializerInterface $serializer, EntityManagerInterface $entityManager, MetadataHelper $metadataHelper, RequestStack $requeststack, PropertyAccessorInterface $propertyAccessor)
    {
        $this->iriConverter = $iriConverter;
        $this->requeststack = $requeststack;
        $this->propertyAccessor = $propertyAccessor;
        $this->inflector = InflectorFactory::create()->build();
        $this->entityManager = $entityManager;
        $this->metadataHelper = $metadataHelper;
        $this->serializer = $serializer;
    }

    public function FileOwnership(ViewEvent $event): void
    {
        $object = $event->getControllerResult();
        $request = $event->getRequest();
        $method = $request->getMethod();
        if (!$method || $method == Request::METHOD_GET || $method == Request::METHOD_DELETE || !$object instanceof BaseEntity) {
            return;
        }
        $request = $event->getRequest();
        $previousData = $request->attributes->get('previous_data');
        $previousOwner = [];
        $entityClass = get_class($object);
        $entityFieldMap = $this->metadataHelper->getEntityFields($entityClass) ?? null;
        if ($entityFieldMap && count($entityFieldMap['fileFields'])) { // Added extra check because of missing type map, can be removed once all entity are mapped 
            foreach ($entityFieldMap['fileFields'] as $key => $value) {
                $camelized = $this->inflector->camelize($value);
                $getMethod = 'get' . $camelized;
                if ($object->{$getMethod}()) {
                    try {
                        $fileObject = $this->iriConverter->getItemFromIri($object->{$getMethod}());
                        // $value = $this->normalize($fileObject, 'jsonld', $context);

                        if ($fileObject && (in_array(Ownable::class, class_uses(get_class($object))) || in_array(UserOwnable::class, class_uses(get_class($object))))) {

                            if ($previousData) {
                                $previousOwner = method_exists($previousData, 'getOwners') ? $previousData->getOwners() : [];
                                $previousUserOwner = method_exists($previousData, 'getUserOwners') ? $previousData->getUserOwners() : [];
                            }

                            $teams = method_exists($object, 'getOwners') ? $object->getOwners() : [];
                            $userOwners = method_exists($object, 'getUserOwners') ? $object->getUserOwners() : [];

                            // Removing Old Team from Current file objects
                            if ($previousOwner && count($previousOwner)) {
                                foreach ($previousOwner as $oldOwner) {
                                    $fileObject->removeOwner($oldOwner);
                                }
                            }

                            // Removing Old User Owners from Current file objects
                            if ($previousUserOwner && count($previousUserOwner)) {
                                foreach ($previousUserOwner as $oldOwner) {
                                    $fileObject->removeUserOwner($oldOwner);
                                }
                            }

                            // Adding current teams to file objects
                            if ($teams && count($teams)) {
                                foreach ($teams as $team) {
                                    $fileObject->addOwner($team);
                                }
                            }

                            // Adding current user owners to file objects
                            if ($userOwners && count($userOwners)) {
                                foreach ($userOwners as $owner) {
                                    $fileObject->addUserOwner($owner);
                                }
                            }

                            $this->entityManager->persist($fileObject);
                            $this->entityManager->flush();
                        }
                    } catch (InvalidArgumentException | \Exception $e) {
                        $value = null;
                    }
                }
            }
        }
    }
}
