<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Core\PicklistName;
use Doctrine\ORM\EntityManagerInterface;
use App\Entity\ModelMetadata\AttribModelMetadata;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class PicklistNameListener
{
    protected $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function preRemove(PicklistName $picklistName)
    {
        $modulesWithPicklist = $this->getModulesForPicklist($picklistName->getName());
        if (count($modulesWithPicklist) > 0) {
            $moduleNames = "";
            foreach ($modulesWithPicklist as $moduleName) {
                $moduleNames .= $moduleName . " , ";
            }
            $moduleList = substr($moduleNames, 0, -3);
            throw new BadRequestHttpException((sprintf("This picklist is being referenced in %s module(s) and hence cannot be deleted. Please make sure to unlink the picklist from all module references before trying to delete it.", $moduleList)));
        }
    }

    protected function getModulesForPicklist($picklistName)
    {
        $moduleNames = [];
        $attribModelMetadataRepository = $this->entityManager->getRepository(AttribModelMetadata::class);
        $attributes = $attribModelMetadataRepository->createQueryBuilder('o')->andwhere("JSON_GET_TEXT(o.dataSource, 'query') like :picklistname ")->setParameter('picklistname', '%\"value\":"' . $picklistName . '"%')->getQuery()->getResult();
        foreach ($attributes as $attribute) {
            $moduleNames[] = $attribute->getSattrib()->getType();
        }
        return $moduleNames;
    }
}
