<?php

namespace App\EventSubscriber;

use App\Entity\Base\BaseEntity;
use Doctrine\DBAL\DBALException;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;

use Symfony\Component\HttpKernel\KernelEvents;
use App\Entity\ModelMetadata\AttributeMetadata;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use App\Entity\ModelMetadata\StagingModelMetadata;
use Doctrine\Inflector\Inflector as LegacyInflector;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class StagingMetaDataSubscriber implements EventSubscriberInterface
{
    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }
    public function onKernelView(ViewEvent $event)
    {
        $newData = $event->getRequest()->attributes->get('data');
        $method = $event->getRequest()->getMethod();
        if (Request::METHOD_GET == $method || Request::METHOD_DELETE == $method || is_null($newData) || !($newData instanceof BaseEntity) || !($newData instanceof StagingModelMetadata)) {
            return;
        }
        $inflector = InflectorFactory::create()->build();
        $moduleName =  $newData->getDescriptions()['singular'];
        $moduleName = preg_replace('/[^a-z0-9\s]+/i', '', $moduleName);
        $escapedModuleName = LegacyInflector::classify($moduleName);
        $escapedSingular = $inflector->singularize($moduleName);
        $escapedPlural = $inflector->pluralize($moduleName);
        if (!isset($newData->getDescriptions()['plural'])) {
            $description = [];
            $description['plural'] = $escapedPlural;
            $description['singular'] = $moduleName;
            $newData->setDescriptions($description);
        }
        if ($newData->getType() == null || $newData->getType() == '') {
            $newData->setType(\strtolower(LegacyInflector::tableize($escapedModuleName)));
        }
        if ($newData->getTableName() == null || $newData->getTableName() == '') {
            $newData->setTableName(\strtolower(LegacyInflector::tableize($escapedModuleName)));
        }


        foreach ($newData->getAttributes() as $key => $value) {
            if ($value->getName() == null || $value->getName() == '') {
                $apiKey = preg_replace('/[^a-z0-9\s]+/i', '', $value->getDescriptions()['singular']);
                $value->setName(LegacyInflector::classify($apiKey));
            }
            if ($value->getFormType() == 'manyToMany' && ($value->getInversedField() == null || $value->getInversedField() == '')) {
                $relatedModel = $value->getDataSource()['model'];
                $value->setInversedField($newData->getType());
                $stagingRepo = $this->entityManager->getRepository(StagingModelMetadata::class);
                $modelMetadata = $stagingRepo->findOneBy(array("type" => $relatedModel));
                if ($modelMetadata) {
                    try {
                        $relatedField = new AttributeMetadata();
                        $relatedField->setDataSource(["model" => $newData->getType()]);
                        $relatedField->setDescriptions(["singular" => $newData->getDescriptions()['singular']]);
                        $relatedField->setDisplayName("{{".$newData->getDescriptions()['singular']."}}");
                        $relatedField->setFormType("manyToMany");
                        $relatedField->setInversedField($value->getName());
                        $relatedField->setName($newData->getType());
                        $relatedField->setType($newData->getType());
                        $relatedField->setCollection(true);
                        $value->setOwnsRelationship(true);
                        $relatedField->setOwnsRelationship(false);
                        $relatedField->setPeerReplicable(true);
                        $relatedField->setVisibility(true);
                        $relatedField->setReadable(true);
                        $relatedField->setHtmlEscape(false);
                        $relatedField->setOrderIndex(count($modelMetadata->getAttributes()));
                        $modelMetadata->addAttribute($relatedField);
                        $this->entityManager->persist($modelMetadata);
                        $this->entityManager->flush();
                    } catch (\Exception | BadRequestHttpException $e) {
                        throw new BadRequestHttpException("Field name " . $newData->getDescriptions()['singular'] . " already exists on the module " . $modelMetadata->getDescriptions()['singular'] . ". Remove the existing field.");
                    }
                }
            }
        }
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['onKernelView', 67]
        ];
    }
}
