<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;

use App\Providers\PermissionsDictionaryBuilder;
use App\Providers\PermissionsDictionaryProvider;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;


class PermissionResolver
{
    /** @var PermissionsDictionaryProvider */
    protected $permissionsDictionaryProvider;

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /**
     * PermissionResolver constructor.
     * @param PermissionsDictionaryProvider $permissionsDictionaryProvider
     */
    public function __construct(PermissionsDictionaryProvider $permissionsDictionaryProvider, TokenStorageInterface $tokenStorage)
    {
        $this->permissionsDictionaryProvider = $permissionsDictionaryProvider;
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @param string $permission Either {create|read|update|delete}.{module} or {read|write}.{module}.{field}
     * @return bool
     */
    public function resolvePermission($permission, $actor=null)
    {
        if (is_null($actor)) {
            $actor = $this->tokenStorage->getToken()->getUser();
        }
        else if (!is_null($this->tokenStorage->getToken())) {
            return false;
        }
        $permissionsDictionary = $this->permissionsDictionaryProvider->getDataForActor($actor);

        $isGranted = false;
        $delimiterCount = substr_count($permission, '.');
        if ($delimiterCount == 1) {
            list($action, $module) = explode('.', $permission, 2);
        } elseif ($delimiterCount == 2) {
            list($action, $module, $field) = explode('.', $permission, 3);
        }
        if (!array_key_exists($module, $permissionsDictionary)) {
            return false;
        }
        $modulePermissions = $permissionsDictionary[$module];

        if ($delimiterCount === 1) {
            // CRUD action
            $isGranted = $modulePermissions[$action] ?? false;
        } else if ($delimiterCount === 2) {

            $fieldsDictionary = $modulePermissions['fields'];
            $fieldPermissions = array_key_exists($field, $fieldsDictionary) ? $fieldsDictionary[$field] : [];
            $isGranted = (!array_key_exists($action, $fieldPermissions) || $fieldPermissions[$action]) && $this->hasModulePermission($modulePermissions, $action);
        }

        return $isGranted;
    }

    /**
     * @param $modulePermssions
     * @param $action
     * @return bool
     */
    private function hasModulePermission(array $modulePermssions, $action)
    {
        if ($modulePermssions === null || empty($modulePermssions) || $action === "" || $action === null) {
            return false;
        }
        if ($action === PermissionsDictionaryBuilder::PERM_FIELD_WRITE) {
            return (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_CREATE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_CREATE] : false) || (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_UPDATE] : false) || (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_DELETE] : false);
        } else {
            return array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_READ, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_READ] : false;
        }
    }
}
