<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;

use App\Entity\Actors\Actor;
use App\Constants\CacheConstants;
use App\Entity\Authorization\Role;
use App\Providers\PermissionsDictionaryBuilder;
use App\Service\AbstractUserDataProvider;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class PermissionsDictionaryProvider extends AbstractUserDataProvider
{
    protected $cache;

    /** @var PermissionsDictionaryBuilder */
    private $permissionsDictionaryBuilder;

    public function __construct(PermissionsDictionaryBuilder $permissionsDictionaryBuilder, TagAwareCacheInterface $cache, TokenStorageInterface $tokenStorage)
    {
        $this->permissionsDictionaryBuilder = $permissionsDictionaryBuilder;
        $this->cache = $cache;
        parent::__construct($tokenStorage);
    }
    /**
     * @inheritdoc
     */
    public function getDataForActor(Actor $actor)
    {
        $username = $actor->getUserId();
        $item = $this->cache->getItem($username . "_permissions");
        if (!$item->isHit()) {
            /** @var Role[] $roles */
            $roles = $actor->getRoles(false, false);
            $permissionsDictionary = $this->permissionsDictionaryBuilder->buildPermissionsDictionary($roles);
            $item->set($permissionsDictionary);
            // Don't hardcode
            $item->expiresAfter(600);
            $item->tag(CacheConstants::USER_PERMISSIONS);
            $this->cache->save($item);
            return $permissionsDictionary;
        }
        return $item->get();
    }
}
