<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Serializer\Normalizer;

use DateTime;
use Symfony\Component\Serializer\Normalizer\CacheableSupportsMethodInterface;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;

class DateTimeNormalizer implements NormalizerInterface, DenormalizerInterface, CacheableSupportsMethodInterface
{
    private $normalizer;
    const FORMAT_JSON_LD = 'jsonld';
    const FORMAT_JSON = 'json';


    public function __construct(ObjectNormalizer $normalizer)
    {
        $this->normalizer = $normalizer;
    }

    public function normalize($object, $format = null, array $context = [])
    {
        return $object->getTimestamp();
    }

    public function supportsNormalization($data, $format = null): bool
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && $data instanceof \DateTime;
    }

    public function hasCacheableSupportsMethod(): bool
    {
        return true;
    }

    public function supportsDenormalization($data, $type, $format = null)
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && 'DateTime' === $type;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        $date = new DateTime();
        $date->setTimestamp($data);
        return $date;
    }
}
