<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\Core\ImportJob;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Container\ContainerInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Class ImportOptionsGenerator
 * @package App\Service\ConfigExportImport
 */
class ImportOptionsGenerator extends ConfigPorter
{
    /** @EntityManagerInterface */
    private $entityManager;

    public function __construct(EntityManagerInterface $entityManager, ContainerInterface $container)
    {
        $this->entityManager = $entityManager; 
        parent::__construct($container);
    }
    /**
     * Generates all options for import given the config object
     *
     * @param array $config
     * @return array
     */
    public function generate(array $fullConfig, ImportJob $importJob, SymfonyStyle $io, $tokenType) {
        $options = [];
        $increment = intval(90 / count($this->configTypes));
        foreach ($this->configTypes as $key => $service) {
            $config = array_key_exists($key, $fullConfig) ? $fullConfig[$key] : [];
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $portingConfig->setIO($io);
            $io->writeln(sprintf('Working on options for %s', $key));
            $importOptions = $config ? $portingConfig->getImportOptions($fullConfig, $key, $tokenType) : [];
            if (count($importOptions) > 0) {
                $options[$key] = [
                    "include" => true,
                    "values" => $importOptions
                ];
            }
            $importJob
                ->setProgressPercent($importJob->getProgressPercent() + $increment);
            
            $this->entityManager->persist($importJob);
            $this->entityManager->flush();
        }
        if (count($options) == 0) {
            throw new \Exception('No importable configurations were found in the JSON file.');
        }
        return $options;
    }

    /**
     * Same as generate, but uses a folder path which is created from the tar.gz file
     *
     * @param string $folderPath
     * @param ImportJob $importJob
     * @param [type] $tokenType
     * @return void
     */
    public function generateFromFolder(string $folderPath, ImportJob $importJob, SymfonyStyle $io,  $tokenType) {
        $options = [];
        $increment = intval(90 / count($this->configTypes));
        foreach ($this->configTypes as $key => $service) {
            
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $portingConfig->setIO($io);
            $io->writeln(sprintf('Working on options for %s', $key));
            $importOptions = $portingConfig->getImportOptionsFromFile($key, $folderPath, $tokenType);
            if (count($importOptions) > 0) {
                $options[$key] = [
                    "include" => true,
                    "values" => $importOptions
                ];
            }
            $importJob
                ->setProgressPercent($importJob->getProgressPercent() + $increment);
            
            $this->entityManager->persist($importJob);
            $this->entityManager->flush();
        }
        if (count($options) == 0) {
            throw new \Exception('No importable configurations were found in the JSON file.');
        }
        return $options;
    }

}
