<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use ApiPlatform\Core\Api\IriConverterInterface;
use App\Command\ImportJobCommand;
use App\Entity\Core\ImportJob;
use Symfony\Component\Console\Style\SymfonyStyle;
use Psr\Container\ContainerInterface;

/**
 * Class ImportService
 */
class ImportService extends ConfigPorter
{

    protected $iriConverter;

    public function __construct(ContainerInterface $container, IriConverterInterface $iriConverter)
    {
        $this->iriConverter = $iriConverter; 
        parent::__construct($container);
    }

    /**
     * 
     *
     * @param array $config
     * @return array
     */
    public function importConfiguration(array $fullData, ImportJob $importJob, ImportJobCommand $importJobCommand, SymfonyStyle $io, $tokenType='JWT') {
        $options = $importJob->getOptions();
        foreach ($this->configTypes as $key => $service) {
            $data = array_key_exists($key, $fullData) ? $fullData[$key] : [];
            $individualOptions = array_key_exists($key, $options) ? $options[$key] : [];
            if (!$data || count($data) == 0 || !$individualOptions || !$individualOptions['include']) {
                continue;
            }
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $portingConfig->setIO($io);
            $io->writeln(sprintf('Working on %s', $key));
            $importJobCommand->updateImportJob($importJob, null, null, null, null, $key);
            $success = $portingConfig->import($data, $individualOptions['values'], $tokenType);
            if (!$success) {
                return [
                    "success" => false
                ];
            }
        }

        return [
            "success" => true
        ];
    }

    /**
     * Same as import, but uses a folder path which is created from the tar.gz file
     *
     * @param string $folderPath
     * @param ImportJob $importJob
     * @param [type] $tokenType
     * @return void
     */
    public function importConfigurationFromFolder(string $folderPath, ImportJob $importJob, ImportJobCommand $importJobCommand, SymfonyStyle $io, $tokenType='JWT') {
        $options = $importJob->getOptions();
        foreach ($this->configTypes as $key => $service) {
            $individualOptions = array_key_exists($key, $options) ? $options[$key] : [];
            if (!$individualOptions || !$individualOptions['include']) {
                continue;
            }
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $portingConfig->setIO($io);
            $io->writeln(sprintf('Working on %s', $key));
            $importJobCommand->updateImportJob($importJob, null, null, null, null, $key);
            $user = $importJob->getModifyUser();
            $userIri = $this->iriConverter->getIriFromItem($user);
            $portingConfig->setImportUser($userIri);
            $success = $portingConfig->importFromFile($individualOptions['values'], $key, $folderPath, $tokenType);
            if (!$success) {
                return [
                    "success" => false
                ];
            }
        }

        return [
            "success" => true
        ];
    }

}
