<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use App\Constants\AttributeLevelOperators;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

class NormalizerUtilityService
{
    protected $requestStack;

    protected $allowedAttributes = [];

    const POSTMAN_SECRET_KEY = 'eW1hSe7mnlIi1a0mmgno1nSe9Rea7pphG2oAdmCeBAoduYnd';

    public function __construct(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    public function validateRequestKey($context)
    {
        $request = $this->requestStack->getCurrentRequest();
        if ($request) {
            $postman_secret_key = self::POSTMAN_SECRET_KEY;
            if (!$request->query->has("\$secretKey")) {
                return false;
            } else {
                $SECRET_KEY = $request->query->get("\$secretKey");
                if ($SECRET_KEY === $postman_secret_key) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getResponseIdentifier(&$context)
    {
        $relationships = $this->getOption('relationships', $context);
        $export = $this->getOption('export', $context);
        $asJson = $this->getOption('asJson', $context);
        $showPassword = (array_key_exists('showPassword', $context) && $context['showPassword'] === true) ? $context['showPassword'] : $this->validateRequestKey($context);
        if ($showPassword === true) {
            $context['showPassword'] = true;
        }
        $hideUuid = $this->getOption('hideUuid', $context) ?? false;
        return [$relationships, $export, $asJson, $showPassword, $hideUuid];
    }

    private function getSelectedFields()
    {
        $request = $this->requestStack->getCurrentRequest();
        if ($request) {
            if ($request->getMethod() == Request::METHOD_POST) {
                $requestBody = json_decode($request->getContent(), true);
                if (isset($requestBody['__selectFields']) && is_array($requestBody['__selectFields'])) {
                    // Adding UUID as for normalization of relation required minimal of one context to match
                    return array_merge($requestBody['__selectFields'], ["uuid", "id"]);
                } else if (array_key_exists("__selectFields", $request->query->all())) {
                    return array_merge(explode(',', $request->query->get("__selectFields")), ["uuid", "id"]);
                }
            } else if ($request->getMethod() == Request::METHOD_GET && array_key_exists("__selectFields__", $GLOBALS)) {
                return array_merge(explode(',', $GLOBALS['__selectFields__']), ["uuid", "id"]);
            }
        }

        return null;
    }

    public function manipulateContextGroup($context, $relationships = false, $export = false, $asJson = false, $showPassword = false)
    {
        if ($relationships) {
            $context["groups"] = array_key_exists("groups", $context) ? ["fsr_primary"] : ["fsr_all"];
        } else {
            $context["groups"] = ["fsr_primary"];
        }

        if ($showPassword) {
            $context["groups"] = array_merge($context["groups"], ["password"]);
        }

        if ($this->getSelectedFields() && count($context["object_class"]) == 1) {
            $context["groups"] = $this->getSelectedFields();
        }

        return $context;
    }

    /**
     * @param $optionName
     * @param array $context
     * @param bool $default
     * @param bool $validateBoolean
     * @return bool|mixed
     */
    protected function getOption($optionName, array $context, $default = false, $validateBoolean = true)
    {
        $option = $default;
        $request = $this->requestStack->getCurrentRequest();
        if (!is_null($request)) {
            $key = AttributeLevelOperators::OPERATOR_TOKEN . $optionName;
            $rawOption = $request->query->get($key);
            if (is_null($rawOption)) {
                $rawOption = $request->request->get($key, $default);
            }
            $option = $rawOption;
            if ($validateBoolean) {
                $option = filter_var($rawOption, FILTER_VALIDATE_BOOLEAN);
            }
        }

        $option = array_key_exists($optionName, $context)
            ? $context[$optionName]
            : $option;

        return $option;
    }
}
