<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;


use App\Traits\Indexable;
use App\Traits\PeerReplicable;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Util\ClassUtils;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\Common\Persistence\ManagerRegistry;

class IndexableListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    protected $container;

    private $_allocationSize = 1;

    protected $sequenceGenerator;

    /**
     * @var int
     */
    private $_nextValue = 0;
    private $_maxValue = null;

    public function generate(EntityManager $em, $sequence_name)
    {
        if ($this->_maxValue === null || $this->_nextValue == $this->_maxValue) {
            // Allocate new values
            $conn = $em->getConnection();
            $sql  = $conn->getDatabasePlatform()->getSequenceNextValSQL($sequence_name);
            $this->_nextValue = (int) $conn->fetchOne($sql);
            $this->_maxValue  = $this->_nextValue + $this->_allocationSize;
        }

        return $this->_nextValue++;
    }

    public function isPeerReplicable($entity_class)
    {
        return in_array(PeerReplicable::class, class_uses($entity_class));
    }

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isInstanceOfIndexableEntity($object)) {
            /** @var BaseEntity $object */
            $manager = $this->managerRegistry->getManagerForClass(get_class($object));
            if ($manager instanceof EntityManager) {
                if ($this->container->hasParameter('id_per_module') && $this->container->getParameter('id_per_module')) {
                    $className = ClassUtils::getRealClass(get_class($object));
                    $tableName = $manager->getClassMetadata($className)->getTableName();
                    $seq_name = $tableName . '_id_seq';
                    $id = $this->generate($manager, $seq_name);
                } else {
                    $id = $this->sequenceGenerator->generate($manager, $object);
                }
                $object->setId($id);
                if ($this->isPeerReplicable($object) && is_null($object->getTenantRecordId())) {
                    $object->setTenantRecordId($id);
                }
            }
        }
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isInstanceOfIndexableEntity($object)
    {
        return in_array(Indexable::class, class_uses(get_class($object)));
    }


    /**
     * Get the value of container
     */
    public function getContainer()
    {
        return $this->container;
    }

    /**
     * Set the value of container
     *
     * @return  self
     */
    public function setContainer($container)
    {
        $this->container = $container;

        return $this;
    }

    /**
     * @param SequenceGenerator $sequenceGenerator
     */
    public function setSequenceGenerator($sequenceGenerator)
    {
        $this->sequenceGenerator = $sequenceGenerator;
    }
}
