<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Validator;

use Doctrine\ORM\EntityManagerInterface;
use App\Entity\ModelMetadata\ModelMetadata;
use Symfony\Component\Validator\Constraint;
use App\Validator\DescriptionNameConstraint;
use App\Entity\ModelMetadata\StagingModelMetadata;
use Symfony\Component\Validator\ConstraintValidator;

class DescriptionNameConstraintValidator extends ConstraintValidator
{
    protected $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function validate($value, Constraint $constraint)
    {
        /* @var $constraint \App\Validator\SingularName */
        if (null === $value || '' === $value || !$constraint instanceof DescriptionNameConstraint) {
            return;
        }
        $modelMetadataRepository = $this->entityManager->getRepository(StagingModelMetadata::class);
        $stagingData = $modelMetadataRepository->findAll();
        foreach ($stagingData as $modelMetadata) {
            $singularName = $modelMetadata->getDescriptions()['singular'];
            $type = $modelMetadata->getType();
            if ($value->getDescriptions()['singular'] == $singularName && $value->getType() != $type) {
                $this->context->buildViolation($constraint::MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getDescriptions()['singular']))
                    ->addViolation();
                break;
            }
        }
    }
}
